// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OButtonBar.hpp

#ifndef OBUTTONBAR_INCLUDED
  #define OBUTTONBAR_INCLUDED


#ifndef OFRAME_INCLUDED
  #include <OFrame.hpp>
#endif

#ifndef OBITMAP_INCLUDED
  #include <OBitmap.hpp>
#endif

#ifndef OICON_INCLUDED
  #include <OIcon.hpp>
#endif

/* 
  The OButtonBar sends a WM_CONTROL-Message to the parent window
  if the mouse pointer is moved over one of the buttons
  this message looks like
   
  WinSendMsg(parent, WM_CONTROL,
             MPFROM2SHORT(OBUTTONBAR_MOVEOVER, buttonID), MPARAM(helptext));

  Process this message, if you want to display the helptext somewhere.
*/

#define OBUTTONBAR_MOVEOVER  WM_MOUSEMOVE

/*
  The OButtonBar sends a WM_CONTROL-Message to the parent window
  if a mouse button is pressed down on one of the buttons
  this message looks like

  WinSendMsg(parent, WM_CONTROL,
             MPFROM2SHORT(OBUTTONBAR_BUTTONxDOWN, buttonID), NULL);

  x means the pressed button. Process this message, if you want to eg. pupup a menu.
*/


#define OBUTTONBAR_BUTTON1DOWN  WM_BUTTON1DOWN
#define OBUTTONBAR_BUTTON2DOWN  WM_BUTTON2DOWN
#define OBUTTONBAR_BUTTON3DOWN  WM_BUTTON3DOWN


typedef class OButtonBar *pOButtonBar;

typedef struct _ButtonBarInfo
{
 ULONG     id;
 ULONG     res;
 PSZ       helptext;
 PSZ       file;
 HWND      hwnd;
 OBitmap   *bmp;
 OIcon     *ico;
} ButtonInfo, *pButtonInfo;


typedef List<ButtonInfo> buttonList;
typedef buttonList *pbuttonList;


class __CPP_EXPORT__ OButtonBar : public OFrame
{
 protected:
    void  initButtonBarDefaults();

 private:
    ULONG resType;
    ULONG orientation;
    ULONG bHeight;
    ULONG bWidth;
    ULONG lastHelpRequest;
    ULONG picSize;

 public:
    buttonList buttons;    // linked list of buttoninfo


    enum {
       iconlike = 0,       // buttons use icons
       bitmaplike,         // buttons use bitmaps
       up2down,            // default
       left2right };


    OButtonBar(const ULONG id,
               const HWND parentWindow, 
               const ULONG resourceType = OButtonBar::iconlike);

    OButtonBar(const OFrame& parentWindow, 
               const ULONG resourceType = OButtonBar::iconlike);

    OButtonBar(const pOFrame parentWindow, 
               const ULONG resourceType = OButtonBar::iconlike);

    virtual ~OButtonBar();

    BOOL createButtonBar();

    void setButtonDimensions(const ULONG height = 32,
                             const ULONG width = 32);
    void setResType(ULONG type = OButtonBar::iconlike);
    void setOrientation(ULONG type);
    void setPicSize(ULONG length);
    void setupButtons();
    void destroyButtons();
    void enableButton(LONG id, BOOL enabled);
    PSZ requestHelpText(const ULONG id, BOOL isHandle); // request help text of a specified button
    PSZ requestHelpText();                              // request help text of the button under the pointer
    ULONG getHeight();
    ULONG getWidth();

    virtual BOOL OCommand(ULONG msg, MPARAM mp1, MPARAM mp2);
};


#endif // OBUTTONBAR_INCLUDED

// end of source
