// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OContainerBase.hpp

#ifndef OCONTAINERBASE_INCLUDED
  #define OCONTAINERBASE_INCLUDED


#ifndef OFRAME_INCLUDED
  #include <oframe.hpp>
#endif

#ifndef OSTRING_INCLUDED
  #include <OString.hpp>
#endif


typedef class OContainerBase *pOContainerBase;

class __CPP_EXPORT__ OContainerBase : public OWindow
{
 public:
   CNRINFO cnri;

//ctors

   OContainerBase(const ULONG id,             // id of the container (must be unique!)
                  const HWND Parent,          // becomes parent AND owner of the container
                  const ULONG cnrStyle,       // container style
                  const ULONG initViewStyle); // default view style

   OContainerBase(const OFrame& Parent,       // becomes parent AND owner of the container
                  const ULONG id,             // id of the container (must be unique!)
                  const ULONG cnrStyle,       // container style
                  const ULONG initViewStyle); // default view style

   OContainerBase(const pOFrame Parent,       // becomes parent AND owner of the container
                  const ULONG id,             // id of the container (must be unique!)
                  const ULONG cnrStyle,       // container style
                  const ULONG initViewStyle); // default view style

   virtual ~OContainerBase();


   BOOL createContainer();                // creates the container and calls setupDetails()
                                          // and setupItems()

   BOOL inherit(const HWND hwndDlg);

   BOOL setView(ULONG viewStyle);
   BOOL setTitle(PSZ title);

   virtual BOOL setupDetails() = 0;       // sets up details view colums
   virtual BOOL setupItems() = 0;         // inserts all records

   BOOL freeDetails();                    // frees details fieldinfo
   BOOL freeItems();                      // removes all items
   BOOL freeContainer();                  // calss freeDetails() and freeItems()


   PFIELDINFO  allocDetailFields(ULONG fields);

   PRECORDCORE _allocRecords(ULONG additional, ULONG records);

// services

   BOOL arrangeRecords();

   BOOL collapseTree(PVOID rootRecord = NULL);

   BOOL expandTree(PVOID rootRecord = NULL);

   BOOL openEdit(PCNREDITDATA editData);

   BOOL closeEdit();

   BOOL eraseRecord(PVOID record);

   BOOL freeDetailInfo(PFIELDINFO rootRecord,
                       ULONG count);

   BOOL freeRecord(PVOID rootRecord,
                   ULONG count);

   BOOL removeDetailInfo(PFIELDINFO rootRecord,
                         ULONG count,
                         ULONG flag = CMA_FREE | CMA_INVALIDATE);

   BOOL removeRecord(PVOID rootRecord,
                     ULONG count, 
                     ULONG flag = CMA_FREE | CMA_INVALIDATE);

   BOOL filter(PFN function, 
               PVOID storage);          // not yet implemented

   BOOL insertDetailInfo(PFIELDINFO info, 
                         PFIELDINFOINSERT insert);

   BOOL insertRecord(PVOID roorRecord,
                     PRECORDINSERT insert);

   BOOL invalidateDetails();

   BOOL invalidateRecord(PVOID rootRecord = NULL,
                         ULONG flag = CMA_REPOSITION,
                         ULONG count = 0);           // invalidate all

   ULONG queryCnrInfo();

   PFIELDINFO queryDetailInfo(PFIELDINFO rootRecord,
                              ULONG flag = CMA_FIRST);

   PRECORDCORE _queryRecord(PVOID rootRecord,
                            ULONG flag = CMA_FIRST,
                            ULONG order = CMA_ITEMORDER);

   PRECORDCORE _queryEmphasis(PVOID record = (PVOID)CMA_FIRST,
                              ULONG flag = CRA_SELECTED);

   BOOL setCnrInfo(ULONG attributeFlag = CMA_FLWINDOWATTR);

   BOOL setEmphasis(PVOID record,
                    ULONG emphasis = CRA_SELECTED,
                    BOOL set = TRUE);
};


#endif // OCONTAINERBASE_INCLUDED

// end of source
