// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OMuxWaitSem.hpp


#ifndef OMUXWAITSEM_INCLUDED
  #define OMUXWAITSEM_INCLUDED


#ifndef OVIOEXCEPTION_INCLUDED
  #include <OVioException.hpp>
#endif


#ifndef ONSEM_INCLUDED
  #include <ONSem.hpp>
#endif

#ifndef OMUXSEM_INCLUDED
  #include <OMuxSem.hpp>
#endif


// in case of any return value !TRUE check muxwaitState for error number


typedef class OMuxWaitSem *pOMuxWaitSem;

class __CPP_EXPORT__ OMuxWaitSem
{
 private:
   HMUX   hmux;
 
 public:
   ULONG  muxwaitState;


 enum OMuxWaitSemActions
  {
   open = 0,
   create
  };

 enum OMuxWaitOptions
  {
   nowait = SEM_IMMEDIATE_RETURN,
   indefinite = SEM_INDEFINITE_WAIT,
   waitAll = DCMW_WAIT_ALL,
   waitAny = DCMW_WAIT_ANY,
   shared = DC_SEM_SHARED
  };

 OMuxWaitSem(HMUX handle);               // throws OVioExceptio if handle is invalid
                                         // or the semaphore cannot be opened

                                         // throws OVioException if the semaphore cannot be
                                         // created or opened
 OMuxWaitSem(PCSZ name,                  // named MuxWaitSem name
             ULONG numSem,               // number of semaphores in semArray
             PSEMRECORD semArray,        // array of semaphores
             ULONG action = OMuxWaitSem::create,
             ULONG attributes = OMuxWaitSem::waitAny | OMuxWaitSem::shared);


 virtual ~OMuxWaitSem();


 ULONG waitMuxWaitSem(ULONG waitFlag = OMuxWaitSem::indefinite);
                                         // let OMuxWaitSem wait
                                         // returns posted or last posted event semaphore

 BOOL openMuxWaitSem(HMUX handle);
 BOOL openMuxWaitSem(PCSZ name);

 BOOL closeMuxWaitSem();

 BOOL addSemaphore(PSEMRECORD record);   // add semaphore to wait list
 BOOL addSemaphore(ONSem *sema);          // add ONSem object to wait list
 BOOL addSemaphore(OMuxSem *sema);        // add OMuxSem object to wait list

 BOOL delSemaphore(HSEM handle);         // delete semaphore from the wait list

 BOOL queryMuxWaitSem(PULONG num,          // return number of semaphores in wait list
                      PSEMRECORD semArray, // return array of semapores in wait list
                      PULONG attributes);  // return creation flags of current OMuxWaitSem

 void setMuxWaitSemHandle(HMUX handle);
 HMUX getMuxWaitSemHandle();
};





#endif // OMUXWAITSEM_INCLUDED

// end of source
