// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// ONPipe.hpp

#ifndef ONPIPE_INCLUDED
  #define ONPIPE_INCLUDED

#ifndef OSTRING_INCLUDED
  #include <OString.hpp>
#endif

#ifndef OTHREAD_INCLUDED
  #include <OThread.hpp>
#endif

#ifndef OLIST_INCLUDED
  #include <OList.hpp>
#endif


#define SVR_PIPE_OPEN_DEFAULT  NP_ACCESS_DUPLEX | NP_NOINHERIT | NP_WRITEBEHIND
#define SVR_PIPE_PIPE_DEFAULT  NP_NOWAIT | NP_READMODE_MESSAGE | NP_TYPE_MESSAGE
#define CLI_PIPE_OPEN_DEFAULT  OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYNONE | \
                               OPEN_FLAGS_WRITE_THROUGH
#define PIPE_TIMEOUT_DEFAULT   1000    // milliseconds till pipe times out


typedef class ONPipe *pONPipe;            // pure base
typedef class OPipeSvr *pOPipeSvr;   // a pipe Server
typedef class OPipeCli *pOPipeCli;     // a pipe Client

typedef struct _PipeStatus
{
 HPIPE pipe;       // pipe handle
 BOOL connected;   // is pipe connected?
} PipeStatus, *pPipeStatus;


typedef List<PipeStatus>  pipelist;
typedef pipelist *ppipelist;

class __CPP_EXPORT__ ONPipe
{
 public:
   ULONG   oMode;
   ULONG   pMode;
   ULONG   timeOut;
   BOOL    close;
   PVOID   Buffer;
   ULONG   BufSize;

   ONPipe();
   virtual ~ONPipe();

   void setOpenMode(const ULONG mode);
   void setPipeMode(const ULONG mode);
   void setTimeOut(const ULONG timeout);

// pure virtuals

   virtual BOOL beginPiping(PCSZ pipeName) = 0;
   virtual BOOL stopPiping() = 0;
   virtual BOOL postPipe(PVOID pvData, ULONG client) = 0;
};


class __CPP_EXPORT__ OPipeSvr : public ONPipe
{
 private:
   ULONG inBuf;
   ULONG outBuf;
   ULONG instances;

   BOOL  createPipe(PCSZ pipeName);
   BOOL  destroyPipe();

 public:
   OThread<OPipeSvr> pipeThread;
   pipelist          pipes;
   pPipeStatus       actual;
   ULONG             connectedClients;


   OPipeSvr(const ULONG inSize,
            const ULONG outSize,
            const ULONG Size,
            const ULONG openMode = SVR_PIPE_OPEN_DEFAULT,
            const ULONG pipeMode = SVR_PIPE_PIPE_DEFAULT,
            const ULONG timeout = PIPE_TIMEOUT_DEFAULT,
            const ULONG inst = 1);

   virtual ~OPipeSvr();

   void setBufSizes(const ULONG in, const ULONG out, const ULONG Size);

   BOOL beginPiping(PCSZ pipeName);
   BOOL stopPiping();
   BOOL getPipeData();
   BOOL postPipe(PVOID pvData,
                 ULONG client = 0); // default: post to all connected clients

// override this
   virtual void OPipeCommand(PVOID pvData) = 0;

// threads

   void Pipe_IO();
};


class __CPP_EXPORT__ OPipeCli : public ONPipe
{
 private:
   BOOL waitForInstance;

   BOOL connectPipe(PCSZ pipeName);
   BOOL disconnectPipe();
 
 public:
   OThread<OPipeCli> pipeThread;
   OString           Name;
   HPIPE             Pipe;
   BOOL              connected;

   OPipeCli(const ULONG Size,
            const ULONG openMode = CLI_PIPE_OPEN_DEFAULT,
            const ULONG pipeMode = SVR_PIPE_PIPE_DEFAULT,
            const ULONG timeout = PIPE_TIMEOUT_DEFAULT,
            const BOOL  waitFor = FALSE);         // if this is TRUE
                                                  // beginPiping() will return only
                                                  // when a pipe instance became available
                                                  // default is to return FALSE when no
                                                  // pipe instance could be opened

   virtual ~OPipeCli();

   void setSize(const ULONG Size);
   BOOL beginPiping(PCSZ pipeName);
   BOOL stopPiping();
   BOOL postPipe(PVOID pvData,
                 ULONG client = 0); // unused, for compatibility

// override this
   virtual void OPipeCommand(PVOID pvData) = 0;

// threads
   void Pipe_IO();
};



#endif // OPIPE_INCLUDED

// end of source
