// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// ONSem.hpp    
// Named Semaphores

 
#ifndef ONSEM_INCLUDED
  #define ONSEM_INCLUDED


#ifndef OSTRING_INCLUDED
  #include <OString.hpp>
#endif

#ifndef OVIOEXCEPTION_INCLUDED
  #include <OVioException.hpp>
#endif

typedef class ONSem *pONSem;


class __CPP_EXPORT__ ONSem
{
 private:
   OString semName;
   HEV     sema;
   ULONG   semState;

 public:

 enum ONSemStates 
  {
   indefinite = SEM_INDEFINITE_WAIT,
   immediate = SEM_IMMEDIATE_RETURN 
  };

 enum ONSemCtorActions 
  {
   create = 0,                   // constructor should try to create the semaphore
   open                          // constructor should try to open the semaphore
  };


   ONSem(HEV handle);            // construct from an existing semaphore
                                 // throws OVioException if the semaphore 
                                 // cannot be opened

// if name is NULL the semaphore is created unnamed

   ONSem(PCSZ name = NULL,       // throws OVioException in case of any failure
         ULONG action = ONSem::create,
         BOOL initialState = FALSE); 


   virtual ~ONSem();


// createSem() was removed. Now the ctors create/open
// the semaphore and throw an OVioException
// in case of any failure.

   BOOL waitSem(const ULONG timeout = ONSem::indefinite);
                                 // wait for event for timeout milliseconds
   BOOL postSem();               // post the semaphore
   BOOL closeSem();              // close the semaphore

   BOOL openSem(PCSZ name);      // open existing semaphore by name
   BOOL openSem(const HEV aSem); // open existing semaphore by handle

   ULONG resetSem();             // reset semaphore, returns postCount since last reset
   ULONG querySem();             // query semaphore, returns postCount

   void setSemHandle(HEV hev);
   HEV  getSemHandle();          // retrieve sema

// setSemName(PCSZ name) was removed because meaningless
};


#endif // ONSEM_INCLUDED

// end of source
