// OCL - Class Library for OS/2 2.x and up
// (c) 1994 Matthias Stbner
// (c) Benjamin Stein 1994
// All Rights Reserved
// OSlider.hpp


#ifndef OSLIDER_INCLUDED
  #define OSLIDER_INCLUDED


#ifndef OFRAME_INCLUDED
  #include <OFrame.hpp>
#endif


/* Mgliche Styles fr den Slider sind:

- SLS_HORIZONTAL    - Slider ist horizontal
- SLS_VERTICAL      - Slider ist vertikal
- SLS_CENTER        -
- SLS_BOTTOM        -
- SLS_TOP           -
- SLS_LEFT          -
- SLS_RIGHT         -

- SLS_PRIMARYSCALE1 - Slider mit oberer Scale
- SLS_PRIMARYSCALE2 - Slider mit unterer Scale
    (wenn beide Parameter zusammen angegeben werden wird vom System
     die untere Scale genommen)

- SLS_HOMELEFT      - Anfangspkt. links (nur horiz. Slider)
- SLS_HOMERIGHT     - Anfangspkt. rechts (nur horiz. Slider)
- SLS_HOMEBOTTOM    - Anfangspkt. unten (nur vert. Slider)
- SLS_HOMETOP       - Anfangspkt. oben (nur vert. Slider)
- SLS_BUTTONSLEFT   - Controllbuttons links (nur horiz. Slider)
- SLS_BUTTONSRIGHT  - Controllbuttons rechts (nur horiz. Slider)
- SLS_BUTTONSBOTTOM - Controllbuttons unten (nur vert. Slider)
- SLS_BUTTONSTOP    - Controllbuttons oben (nur vert. Slider)
- SLS_SNAPTOINCREMENT    - Marker springt immer zum nchsten GANZEN Wert
- SLS_READONLY      - entfernt Schieber und Controllbuttons (z.B. Progressind.)
- SLS_RIBBONSTRIP   - zeichnet Slider zw. Anfang und Schieber mit Farbe aus
- SLS_OWNERDRAW     - wenn man unbedingt selber malen mchte ;-))

*/

typedef class OSlider *pOSlider;

class __CPP_EXPORT__ OSlider : public OWindow
{
 protected:
    ULONG     Scale1;
    ULONG     Space1;
    ULONG     Scale2;
    ULONG     Space2;
    SLDCDATA  ctlData;

 public:

// ctors
    OSlider(const ULONG ID,
            const HWND  Parent,  // becomes parent and owner of the slider
            const ULONG scale1,
            const ULONG space1,
            const ULONG scale2,
            const ULONG space2,
            const ULONG Style);

    OSlider(const OFrame& Parent,
            const ULONG scale1,
            const ULONG space1,
            const ULONG scale2,
            const ULONG space2,
            const ULONG Style);

    OSlider(const pOFrame Parent,
            const ULONG scale1,
            const ULONG space1,
            const ULONG scale2,
            const ULONG space2,
            const ULONG Style);

// dtor
    virtual ~OSlider();


    BOOL createSlider(const SHORT x  = 0,
                      const SHORT y  = 0,
                      const SHORT cx = 0, 
                      const SHORT cy = 0);


    BOOL inherit(const HWND Parent);

    // setzt style auf bergebenen Wert (noch keinen tieferen Sinn)
    void setSliderStyles(const ULONG Style);

    // setzt sizepos auf neue Werte (noch ohne tieferen Sinn)
    void setSliderSize(const ULONG x, 
                       const ULONG y, 
                       const ULONG cx, 
                       const ULONG cy);

    BOOL setSliderArmPos(SHORT pos);                   //  Setzt die Position des Schiebers
    BOOL setSliderArmSize(ULONG x, ULONG y);           //  Setzt die Grsse des Schiebers
    BOOL setSliderShaftPos(SHORT x, SHORT y);          //  Setzt Position des Schaftes im
                                                       //  Slider-Fenster
    BOOL setSliderShaftDim(SHORT dim);                 //  Setzt die Dicke des Schaftes
    BOOL incSliderArm(SHORT delta = 1);                //  erhht Schieberwert (default = 1)
    BOOL decSliderArm(SHORT delta = 1);                //  verringert Schieberwert (default = 1)

    BOOL setSliderScaleText(PCSZ Text, USHORT pos);    // setzt Text am Tick Nr. pos

    // setzt Grsse der Ticks (default setzt ALLE Ticks auf Grsse 10 Pixel)
    BOOL setTickSize(ULONG size = 10, ULONG num = SMA_SETALLTICKS);

    // fgt 'Unter'tick ein und liefert ID zurck
    ULONG addDetent(SHORT pos);

    // liefert Pos. des durch ID bezeichneten 'Unter'tick
    ULONG getDetentPos(ULONG id);

    // fragt Text an Tick ab
    PSZ getScaleText(SHORT loc, PSZ buf,SHORT len);

    // fragt Grsse des Sliderschaftes ab
    void getSliderShaftDim(SHORT *lenght, SHORT *height);
 
    // fragt die Pos. des Sliderschaftes im Sliderfenster ab
    void getSliderShaftPos(SHORT *x, SHORT *y);

    // fragt Grsse des Schiebers ab
    void getSliderArmSize(SHORT *x, SHORT *y);

    // holt Schieberwert in Scaleneinheiten (NICHT in Pixeln!)
    ULONG getSliderArmPos();

    // fragt Grsse des Ticks an der Pos. pos ab
    ULONG getTickSize(SHORT pos);


// for ownerdrawn sliders
    void drawSliderElement(POWNERITEM  item, COLOR color);
};


#endif // OSLIDER_INCLUDED
