// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OSystem.h



#ifndef OSYSTEMH_INCLUDED
  #define OSYSTEMH_INCLUDED


#define PROCESS_END_INDICATOR   3       /* Indicates end of process structs */
#define PROCSTAT_BUFFER_SIZE    0x4000

#pragma pack(1)

typedef struct _SUMMARY
{
    ULONG   ulThreadCount;              /* Number of threads in system */
    ULONG   ulProcessCount;             /* Number of processes in system */
    ULONG   ulModuleCount;              /* Number of modules in system */
} SUMMARY, *PSUMMARY;

typedef struct _THREADINFO
{
    UCHAR   uchDontKnow1;               
    UCHAR   uchDontKnow2;              
    USHORT  usDontKnow3;               
    USHORT  tidWithinProcess;           /* TID within process (TID is 4 bytes!!) */
    USHORT  tidWithinSystem;            /* TID within system */
    ULONG   ulBlockId;                  /* Block ID (?) */
    USHORT  usPriority;                 /* Priority */
    USHORT  usDontKnow4;                
    USHORT  usDontKnow5;                
    USHORT  usDontKnow6;                
    USHORT  usDontKnow7;                
    USHORT  usDontKnow8;                
    USHORT  usThreadStatus;             /* 2=blocked or ready, 5=running */
    USHORT  usDontKnow9;                
} THREADINFO, *PTHREADINFO;

typedef struct _PROCESSINFO
{
    ULONG       ulEndIndicator;         /* 1 means not end, 3 means last entry */
    PTHREADINFO ptiFirst;               /* Address of the 1st Thread Control Blk */
    USHORT      pid;                    /* Process ID (2 bytes - PID is 4 bytes) */
    USHORT      pidParent;              /* Parent's process ID */
    USHORT      usDontKnow2;            
    USHORT      usDontKnow3;            
    USHORT      usDontKnow4;            
    USHORT      usDontKnow5;            
    USHORT      idSession;              /* Session ID */
    USHORT      usDontKnow6;           
    USHORT      hModRef;                /* Module handle of EXE */
    USHORT      usThreadCount;          /* Number of threads in this process */
    USHORT      usSessionType;          /* Session type (SSF_TYPE_xx) */
    CHAR        achDontKnow7[ 6 ];      
    USHORT      usThreadIdCount;        /* Number of USHORTs in Thread id table? */
    USHORT      usModCount;             /* Number of USHORTs in ModHandle table? */
    USHORT      usUshortCount;          /* Number of USHORTs in Ushort table? */
    USHORT      usDontKnow8;            
    ULONG       ulThreadIdTableAddr;    /* Maybe the address of a TID table */
    ULONG       ulModHandleTableAddr;   /* Address of a ModHandle table (DLLS?)  */
    ULONG       ulUshortTableAddr;      /* Address of a table of USHORTs (?) */
} PROCESSINFO, *PPROCESSINFO;

typedef struct _SEMINFO
{
    struct _SEMINFO *pNext;             /* Pointer to next block */
    USHORT   idOwningThread;            /* ID of owning thread? */
    USHORT   fsFlags;                   /* (MSB-LSB) */
    UCHAR    uchReferenceCount;         /* Number of references */
    UCHAR    uchRequestCount;           /* Number of requests */
    CHAR     achDontKnow1[ 7 ];         
    USHORT   usIndex;                  /* Index (?) */
    CHAR     szSemName[ 1 ];            /* ASCIIZ semaphore name */
} SEMINFO, *PSEMINFO;

typedef struct _SHRMEMINFO
{
    struct _SHRMEMINFO *pNext;           /* Pointer to next block */
    USHORT      usMemHandle;             /* Shared memory handle (?) */
    SEL         selMem;                  /* Selector */
    USHORT      usReferenceCount;        /* Number of references */
    CHAR        szMemName[ 1 ];          /* ASCIIZ shared memory name */
} SHRMEMINFO, *PSHRMEMINFO;

typedef struct _MODINFO
{
    struct _MODINFO *pNext;              /* Pointer to next block */
    USHORT   hMod;                       /* Module handle */
    USHORT   usModType;                  /* Module type (0=16bit,1=32bit) */
    ULONG    ulModRefCount;              /* Count of module references */
    ULONG    ulSegmentCount;             /* Number of segments in module */
    ULONG    ulDontKnow1;                
    PSZ      szModName;                  /* Addr of fully qualified module name */
    USHORT   usModRef[ 1 ];              /* Handles of module references */
} MODINFO, *PMODINFO;

typedef struct _BUFFHEADER
{
    PSUMMARY        psumm;               /* Pointer to SUMMARY section */
    PPROCESSINFO    ppi;                 /* Pointer to PROCESS section */
    PSEMINFO        psi;                 /* Pointer to SEMAPHORE section */
    PVOID           pDontKnow1;          
    PSHRMEMINFO     psmi;                /* Pointer to SHARED MEMORY section */
    PMODINFO        pmi;                 /* Pointer to MODULE section */
    PVOID           pDontKnow2;          
    PVOID           pDontKnow3;          
} BUFFHEADER, *PBUFFHEADER;

#pragma pack()


#endif // OSYSTEMH_INCLUDED

/* end of source */
