// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OSystem.hpp


#ifndef OSYSTEMHPP_INCLUDED
  #define OSYSTEMHPP_INCLUDED

#include <OSystem.h>  // definitions for undocumented API DosQProcStatus()


#ifndef SORTEDLIST_INCLUDED
  #include <OSortedList.hpp>
#endif

// a processes status List

// class definitions

typedef class OProcRecord *pOProcRecord;

class __CPP_EXPORT__ OProcRecord
{
 public:
   OString  Module;  // full qualified module name or DOS-Process for SYSINIT
   ULONG    Threads; // number of threads
   ULONG    Pid;     // Process ID
   ULONG    PPid;    // Parent Process ID
   ULONG    Sid;     //  Session ID


   OProcRecord();
   OProcRecord(const OProcRecord& newRecord);
   OProcRecord(const pOProcRecord newRecord);
   virtual ~OProcRecord();
};
    

// linked List of OProcRecords

typedef class slOProcRecord *pslOProcRecord;

class __CPP_EXPORT__ slOProcRecord : public SortedList<OProcRecord>
{
 public:
  slOProcRecord();
  virtual ~slOProcRecord();

  void operator << (pOProcRecord newRecord); // add an item to the linked List
  void output(PVOID elem);                   // overload this for PM apps

  BOOL isLess(const OProcRecord *first, const OProcRecord *second);
};


// the complete Status Report

typedef class OProcStat *pOProcStat;

class __CPP_EXPORT__ OProcStat
{
 private:
    ULONG getNumInfo(PSZ module, ULONG flag);

 public:
    slOProcRecord  ProcList;
    ULONG          TaskCount;
    ULONG          ProcCount;

   OProcStat();
   virtual ~OProcStat();

   BOOL getProcesses();
   ULONG getPid(PSZ module);    // get a PID from a module name
   ULONG getPPid(PSZ module);   // get a PPID from a module name
   PSZ getModule(ULONG pid);    // get a module name from a PID
   ULONG getProcCount();
   ULONG getTaskCount();
};

 
#endif // OSYSTEMHPP_INCLUDED

// end of source
