// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OThread.hpp


#ifndef OTHREAD_INCLUDED
  #define OTHREAD_INCLUDED

#if (defined ( __IBMCPP__ ) && !defined ( OCL_HPP_INCLUDED ))  // in case of template generation
  #define INCL_DOS
  #define INCL_WIN
  #define INCL_PM
  #include <ocl.hpp>
#endif


#ifndef OPMINIT_INCLUDED
  #include <OPMinit.hpp>
#endif


typedef class OThreadBase *pOThreadBase;

class __CPP_EXPORT__ OThreadBase
{
 private:
   ULONG     stack;
   BOOL      initPM;

 public:
   TID    tid;

   OThreadBase(ULONG stackSize, BOOL forPM);
   virtual ~OThreadBase();

// control
   BOOL run();                             // start thread
   BOOL isRunning();                       // is the thread currently running?
   void stop();                            // stop thread
   void suspend();                         // suspend execution
   void resume();                          // resume execution
   BOOL waitFor();                         // wait for termination of this OThreadBase
   void setPrty(ULONG Class = PRTYC_REGULAR,
                ULONG Delta = 0);          // set priority
 
   virtual void async() = 0;               // this method will be executed

// this member is strictly for internal use only

   static void OTHREADBASE_dispatch(PVOID pvData);
};


#ifdef __IBMCPP__
  typedef void ( _Optlink *OptlinkFnPtr ) ( void * );
#endif


template <class T>
class OThread : public OThreadBase
{
 private:
   void(T::*member)();
   T*        object;
   PFNTHREAD threadFunc;
   PVOID     pArgs;

 public:
   OThread(T* obj,                         // pointer to object
           void(T::*mem)(),                // pointer to member-fn to detach
           ULONG stackSize = 8192,         // stacksize for thread
           BOOL  forPM = TRUE);            // initialize PM?

   OThread(T& obj,                         // reference to object
           void(T::*mem)(),                // pointer to member-fn to detach
           ULONG stackSize = 8192,         // stacksize for thread
           BOOL  forPM = TRUE);            // initialize PM?

   OThread(PFNTHREAD aFunction,
           ULONG stackSize = 8192,
           BOOL forPM = TRUE,
           PVOID pvData = NULL);

   virtual ~OThread();


   void async();


#ifdef __IBMCPP__
   OThread(OptlinkFnPtr aFunction,
           ULONG stackSize = 8192,
           BOOL forPM = TRUE,
           PVOID pvData = NULL);
#endif
};

#endif


// end of source
