// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OBitmap.cpp


#define OINCL_OAPP
#define OINCL_OBITMAP
#define OINCL_OPMEXCEPTION


#include <ocl.hpp>


OBitmap::OBitmap()
  : hbm(NULLHANDLE)
  {}



OBitmap::OBitmap(const HWND bitmapOwner,
                 const ULONG resID, 
                 const HMODULE module, 
                 const ULONG stretchCX, 
                 const ULONG stretchCY)
{
 if ((hbm = GpiLoadBitmap(WinGetPS(bitmapOwner), 
                          module, resID, stretchCX, stretchCY)) == NULLHANDLE)
   throw OPMException((PSZ)NULL, 0, OException::recoverable);
}



OBitmap::OBitmap(const HBITMAP hbmHandle)
  : hbm(hbmHandle)
  {}



OBitmap::OBitmap(const HWND bitmapOwner, PCSZ fileName)
  : hbm(NULLHANDLE)
{
 APIRET              rc = 0;
 PBYTE               Buffer;
 FILESTATUS3         stat;
 fstream             bmpFile;
 PBITMAPFILEHEADER   pbmfh;

 if((!fileName) ||
   ((rc = DosQueryPathInfo(fileName, 1, &stat, sizeof(FILESTATUS3))) != 0) ||
   ((rc = DosAllocMem((PPVOID)&Buffer,  stat.cbFile,
                      PAG_READ | PAG_WRITE | PAG_COMMIT)) != 0))
   throw OPMException((PSZ)NULL, rc, OException::recoverable);
 
 bmpFile.open(fileName, ios::in | ios::binary);
 if((bmpFile.fail()) || (!bmpFile.read(Buffer, stat.cbFile))) {
   DosFreeMem(Buffer);
   throw OPMException((PSZ)NULL, 2, OException::recoverable); }

 bmpFile.close();

 pbmfh = (PBITMAPFILEHEADER)Buffer;
 if ((pbmfh) && (pbmfh->usType !=  BFT_BMAP)) {
   DosFreeMem(Buffer);  
   throw OPMException((PSZ)NULL, 11, OException::recoverable); }
 
 hbm = GpiCreateBitmap(WinGetPS(bitmapOwner), (PBITMAPINFOHEADER2) &pbmfh->bmp,
                      (ULONG) CBM_INIT,
                      (PSZ) pbmfh + pbmfh->offBits,
                      (PBITMAPINFO2) &pbmfh->bmp);
 DosFreeMem(Buffer);
 if (!hbm)
   throw OPMException((PSZ)NULL, 0, OException::recoverable);
}


OBitmap::~OBitmap()
{
 if (hbm)
   GpiDeleteBitmap(hbm);
}


// end of source
