// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OBook.cpp


#define INCL_WIN
#define INCL_PM
#define OINCL_BASE

#include <ocl.hpp>
#include <OMsgs.hpp>
#include <OBook.hpp>

// definitions

#define TAB_WIDTH_MARGIN      10
#define TAB_HEIGHT_MARGIN     6
#define DEFAULT_NB_TAB_HEIGHT 16

#define  NBRes  10000  // resource ID for notebook


OBook::OBook(const ULONG id, const HWND Parent, const ULONG frameStyle)
   : OFrame(id, frameStyle, CS_SIZEREDRAW),
     book(NULLHANDLE),
     bStyle(0),
     dimCX(0),
     dimCY(0)
{
 parent = Parent;
 owner  = Parent;
}

OBook::OBook(const OFrame& Parent, const ULONG frameStyle)
   : OFrame(Parent.res, frameStyle, CS_SIZEREDRAW),
     book(NULLHANDLE),
     bStyle(0),
     dimCX(0),
     dimCY(0)
{
 parent = Parent.hwnd;
 owner  = Parent.hwnd;
}


OBook::OBook(const pOFrame Parent, const ULONG frameStyle)
   : OFrame(Parent->res, frameStyle, CS_SIZEREDRAW),
     book(NULLHANDLE),
     bStyle(0),
     dimCX(0),
     dimCY(0)
{
 parent = Parent->hwnd;
 owner  = Parent->hwnd;
}


OBook::~OBook() {}



BOOL OBook::createBook(PCSZ Text, const ULONG cx, const ULONG cy, const ULONG bookStyle)
{
 winText << (PSZ) Text;
 dimCX = cx;
 dimCY = cy;
 bStyle = bookStyle;
 PNBPAGE pP = Pages.getFirst();

 if (!createFrame("OCL Notebook"))
   return(FALSE);

 if ((cNbk()) && (pP) && ((BOOL) WinSendMsg(book, BKM_TURNTOPAGE, MPFROMLONG(pP->ulPageID), NULL)))
  {
   setFramePos();
   return(TRUE); 
  }

 return(FALSE);
}


// Set Frame of the Book Window

void OBook::setSizePosFrame(const HWND behind, SWP dimensions, const ULONG flag)
{
 POINTL aptl[2];
 SWP    swpScreen;

 aptl[0].x = 0;
 aptl[0].y = 0;
 aptl[1].x = dimCX;
 aptl[1].y = dimCY;

 swpScreen.cx=WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN);
 swpScreen.cy=WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);

 WinMapDlgPoints(HWND_DESKTOP, aptl, 2, TRUE);
 WinSetWindowPos(frame, behind, (swpScreen.cx-(aptl[1].x))/2,
                (swpScreen.cy-(aptl[1].y))/2, aptl[1].x, aptl[1].y,
                 SWP_SIZE | SWP_MOVE | SWP_RESTORE | SWP_ACTIVATE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif




// set dimensions of the tabs

BOOL OBook::setTabDimensions()
{
 FONTMETRICS  fm;
 INT          iSize, iLongestMajText = 0, iLongestMinText = 0;
 PNBPAGE      pP = Pages.getFirst();
 BOOL         Majors=FALSE, Minors=FALSE;

 hps = WinGetPS(book);

 if(!hps)
   return(FALSE);

 memset(&fm, 0, sizeof(FONTMETRICS));
 if(GpiQueryFontMetrics(hps, sizeof(FONTMETRICS), &fm))
    fm.lMaxBaselineExt += (TAB_HEIGHT_MARGIN * 2);
 else
    fm.lMaxBaselineExt = DEFAULT_NB_TAB_HEIGHT + (TAB_HEIGHT_MARGIN * 2);

 while(pP)
  {
   iSize = gSS(pP->szTabText);
   if(pP->usTabType == BKA_MAJOR)
    {
     Majors=TRUE;
     if(iSize > iLongestMajText)
       iLongestMajText = iSize;
    }
   else
    {
     Minors=TRUE;
     if(iSize > iLongestMinText)
       iLongestMinText = iSize;
    }
   pP = Pages.getNext();
  }

 WinReleasePS(hps);

 if(iLongestMajText)
   iLongestMajText += TAB_WIDTH_MARGIN;

 if(iLongestMinText)
   iLongestMinText += TAB_WIDTH_MARGIN;

 if (Majors)
   Majors = (BOOL) WinSendMsg(book, BKM_SETDIMENSIONS,
                                        MPFROM2SHORT(iLongestMajText, (SHORT)fm.lMaxBaselineExt),
                                        MPFROMSHORT(BKA_MAJORTAB));
 else Majors=TRUE;

 if (Minors)
   Minors =(BOOL) WinSendMsg(book, BKM_SETDIMENSIONS,
                                       MPFROM2SHORT(iLongestMinText, (SHORT)fm.lMaxBaselineExt),
                                       MPFROMSHORT(BKA_MINORTAB));
 else
    Minors=TRUE;

 return(Majors && Minors);
}



// add pages to the book

BOOL OBook::addPages(ppageList pagesList)
{
 PNBPAGE pP = pagesList->getFirst();

 while(pP) {
    if (sP(pP))
      Pages.addLast(pP);
    pP = pagesList->getNext(); }

 setTabDimensions();
 
 pP = Pages.getFirst();
 if (pP)
   WinSendMsg(book, BKM_TURNTOPAGE, MPFROMLONG(pP->ulPageID), NULL);
 return(TRUE);
}





BOOL OBook::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
    {
     case WM_SIZE:
       WinSetWindowPos(book, 0, 0, 0, SHORT1FROMMP(mp2), SHORT2FROMMP(mp2), SWP_SIZE | SWP_SHOW );
       break;

     case WM_CONTROL:
       if(SHORT1FROMMP(mp1) == NBRes) {
        switch(SHORT2FROMMP(mp1)) {
          case BKN_PAGESELECTED:
            sNBP((PPAGESELECTNOTIFY) mp2);
            return(TRUE); }}
       return(FALSE);

      // da OWinDefWindowProc das Object bei WM_CLOSE oder WM_DESTROY
      // von unserem Handler abkoppelt und mit dem OApp::NullHandler()
      // assoziiert, kann uns immer nur eine dieser beiden Nachrichten passieren.

     case WM_DESTROY:
     case WM_CLOSE: {
       NBPAGE *temp = Pages.getFirst();

       hideFrame();
       while(temp) {
         if (temp->cmd)
           WinSendMsg(temp->cmd->hwnd, msg, NULL, NULL);
         temp = Pages.getNext(); }
       break; }

     default:
       return(OFrame::OCommand(msg, mp1, mp2));
    }
 return(TRUE);
}


/*********************************************************************************************************/

// internals


// Create the Book

BOOL OBook::cNbk()
{
 BOOL  fSuccess = TRUE;

 book = WinCreateWindow(client, WC_NOTEBOOK, NULL,
                        bStyle, 0, 0, 0, 0,
                        client, HWND_TOP, NBRes, NULL, NULL);
 if(book)
  {
   if(!WinSendMsg(book, BKM_SETNOTEBOOKCOLORS,
                  MPFROMLONG(SYSCLR_FIELDBACKGROUND),
                  MPFROMSHORT(BKA_BACKGROUNDPAGECOLORINDEX)))
     return(FALSE);
   else
     {
      if (!setTabDimensions())
         fSuccess = FALSE;
      else
        {
         PNBPAGE pP=Pages.getFirst();

         while ((pP) && (fSuccess)) {
             fSuccess = sP(pP);
             pP = Pages.getNext(); }
        }
     }
   }
 else fSuccess = FALSE;

 return fSuccess;
}


// Load a dialog and associate to a page

HWND OBook::Assoc(PNBPAGE pnbp, PPAGESELECTNOTIFY ppsn)
{
 HWND hwndDlg = WinLoadDlg(hwnd, hwnd, OWinDefDlgProc, 0, pnbp->idDlg, pnbp->cmd);

 if(hwndDlg)
    {
     if(!WinSendMsg(ppsn->hwndBook, BKM_SETPAGEWINDOWHWND,
                    MPFROMP(ppsn->ulPageIdNew), MPFROMLONG(hwndDlg)))
       {
        WinDestroyWindow(hwndDlg);
        hwndDlg = NULLHANDLE;
       }
    }
 else
    NBError("Error loading page dialog.");

 return(hwndDlg);
}


// get a string size

ULONG OBook::gSS(PCSZ szString)
{
 POINTL aptl[TXTBOX_COUNT];

 if(!GpiQueryTextBox(hps, strlen((PCH)szString), (PCH) szString, TXTBOX_COUNT, aptl))
   return 0;
 else
   return aptl[TXTBOX_CONCAT].x;
}


// setup a book page

VOID OBook::sNBP(PPAGESELECTNOTIFY ppsn)
{
 HWND    hwndDlg;
 PNBPAGE pnbp;

 if (!ppsn) return;

 pnbp = (PNBPAGE) WinSendMsg(ppsn->hwndBook, BKM_QUERYPAGEDATA, MPFROMLONG(ppsn->ulPageIdNew), NULL);

 if(!pnbp)
   return;
 else if(pnbp == (PNBPAGE) BOOKERR_INVALID_PARAMETERS)
   return;

 if(pnbp->fParent)
   {
    ULONG ulPageNew;

    if (ppsn->ulPageIdNew > ppsn->ulPageIdCur)
       ulPageNew = (ULONG) WinSendMsg(ppsn->hwndBook,
                                      BKM_QUERYPAGEID, MPFROMLONG(ppsn->ulPageIdNew),
                                      MPFROM2SHORT(BKA_NEXT, BKA_MINOR));
    else if (ppsn->ulPageIdNew < ppsn->ulPageIdCur)
       ulPageNew = (ULONG) WinSendMsg(ppsn->hwndBook,
                                      BKM_QUERYPAGEID, MPFROMLONG(ppsn->ulPageIdNew),
                                      MPFROM2SHORT(BKA_NEXT, BKA_MINOR));
    else return;

    switch(ulPageNew) {
       case 0:
          return;
       case BOOKERR_INVALID_PARAMETERS:
          NBError("Invalid Page specified.");
          return;
       default:
          WinSendMsg(ppsn->hwndBook, BKM_TURNTOPAGE, MPFROMLONG(ulPageNew), NULL);
          return; }
   }
 else
   {
    hwndDlg = (HWND) WinSendMsg(ppsn->hwndBook, BKM_QUERYPAGEWINDOWHWND,
                                     MPFROMLONG(ppsn->ulPageIdNew), NULL);

    if(hwndDlg == (HWND) BOOKERR_INVALID_PARAMETERS)
      hwndDlg = NULLHANDLE;
    else if(!hwndDlg)
      hwndDlg = Assoc(pnbp, ppsn);

    if(hwndDlg && !pnbp->fParent)
      WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwndDlg, pnbp->idFocus));
   }
 return;
}


// set up a particular page

BOOL OBook::sP(PNBPAGE pP)
{
 ULONG ulPageId;

 ulPageId = (ULONG) WinSendMsg(book, BKM_INSERTPAGE, NULL,
                  MPFROM2SHORT(pP->usTabType | BKA_STATUSTEXTON | BKA_AUTOPAGESIZE, BKA_LAST));
 if(ulPageId)
   {
    pP->ulPageID = ulPageId;
    if ((BOOL) WinSendMsg(book, BKM_SETPAGEDATA, MPFROMLONG(ulPageId), MPFROMP(pP)))
      if ((BOOL) WinSendMsg(book, BKM_SETSTATUSLINETEXT, MPFROMP(ulPageId), MPFROMP(pP->szStatusLineText)))
         return((BOOL) WinSendMsg(book, BKM_SETTABTEXT, MPFROMP(ulPageId), MPFROMP(pP->szTabText)));
      else return(FALSE);
    else return(FALSE);
   }
 else return(FALSE);
}


void OBook::NBError(PCH Msg)
{
  WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PCH) Msg,
     "Application Error", 0,
     MB_OK | MB_APPLMODAL | MB_MOVEABLE | MB_CUAWARNING);
}


/********************************************************************************************************/


// end of source
