// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OButtonBar.cpp



#define OINCL_BASE

#include <ocl.hpp>
#include <OButtonBar.hpp>

OButtonBar::OButtonBar(const ULONG id, const HWND parentWindow, const ULONG resourceType)
  : OFrame(id, FCF_NOBYTEALIGN, CS_SIZEREDRAW | CS_HITTEST)
{
 setParent(parentWindow);
 setOwner(parentWindow);
 initButtonBarDefaults();
 resType = resourceType;
 orientation = left2right;
}


OButtonBar::OButtonBar(const OFrame& parentWindow, const ULONG resourceType)
  : OFrame(parentWindow.res, FCF_NOBYTEALIGN, CS_SIZEREDRAW | CS_HITTEST)
{
 setParent(parentWindow.hwnd);
 setOwner(parentWindow.hwnd);
 initButtonBarDefaults();
 resType = resourceType;
 orientation = left2right;
}

OButtonBar::OButtonBar(const pOFrame parentWindow, const ULONG resourceType)
  : OFrame(parentWindow->res, FCF_NOBYTEALIGN, CS_SIZEREDRAW | CS_HITTEST)
{
 setParent(parentWindow->hwnd);
 setOwner(parentWindow->hwnd);
 initButtonBarDefaults();
 resType = resourceType;
 orientation = left2right;
}



OButtonBar::~OButtonBar() {}


void OButtonBar::initButtonBarDefaults()
{
 bHeight = 40;
 bWidth = 40;
 picSize = 32;
 pparms.Fore = SYSCLR_WINDOWTEXT;
 pparms.Back = SYSCLR_WINDOW;
 strcpy(pparms.Font, "8.Helv");
 lastHelpRequest = 0;
}


BOOL OButtonBar::createButtonBar()
{
 if (!createFrame("OCL ButtonBar"))
   return(FALSE);
 else {
   setupButtons();
   return(TRUE); }
}


void OButtonBar::destroyButtons()
{
 if (!buttons.isEmpty()) {
   pButtonInfo tmp = buttons.getFirst();

   while(tmp) {
      WinDestroyWindow(tmp->hwnd);
      tmp = buttons.getNext(); }}
}


void OButtonBar::enableButton(LONG id, BOOL enabled)
{
 pButtonInfo tmp;

 if (!buttons.isEmpty()) {
   tmp  = buttons.getFirst();
   while(tmp) {
     if ((tmp->id == id) || (id == 0))
       WinEnableWindow(tmp->hwnd, enabled);
     tmp = buttons.getNext(); }}
}



PSZ OButtonBar::requestHelpText(const ULONG id, BOOL isHandle)
{
 if (!buttons.isEmpty()) {
   pButtonInfo tmp = buttons.getFirst();

   while(tmp) {
      if (((isHandle) && (tmp->hwnd == id)) || ((!isHandle) && (tmp->id == id)))
        return(tmp->helptext);
      tmp = buttons.getNext(); }}
 return(NULL);
}


PSZ OButtonBar::requestHelpText()
{
 HWND  handle = NULLHANDLE;

 return(requestHelpText(handle, TRUE));
}


void OButtonBar::setupButtons()
{
 CHAR        num[10];
 ULONG       i = 0, buttonFrameStyle, posx, posy;
 pButtonInfo tmp = buttons.getFirst();

 lastHelpRequest = 0;
 hideFrame();

 if (orientation == up2down)
   WinSetWindowPos(frame, 0, 0, 0, bWidth, bHeight*buttons.items, SWP_SIZE);
 else
   WinSetWindowPos(frame, 0, 0, 0, bWidth*buttons.items, bHeight, SWP_SIZE);

 while(tmp) 
  {
   switch(orientation) {
     case up2down:
       posx = 0;
       posy = (buttons.items - i - 1)*bHeight;
       break;
     case left2right:
       posx = i*bWidth;
       posy = 0;
       break; }
    
   if (!tmp->file)
    {
     if (resType == iconlike)
       buttonFrameStyle = BS_NOPOINTERFOCUS | BS_PUSHBUTTON | BS_ICON;
     else
       buttonFrameStyle = BS_NOPOINTERFOCUS | BS_PUSHBUTTON | BS_BITMAP;
     sprintf(num, "#%d", tmp->res);
     tmp->hwnd = WinCreateWindow(client, WC_BUTTON, num, buttonFrameStyle,
                                 posx, posy, bWidth, bHeight,
                                    client, HWND_TOP, tmp->id, NULL, NULL); 
    }
   else  
    {
     buttonFrameStyle = BS_NOPOINTERFOCUS | BS_PUSHBUTTON | BS_USERBUTTON;
     switch(resType) {
       case iconlike:
        try {
          tmp->ico = new OIcon(tmp->file); }
        catch (...) {
          delete tmp->ico;
          tmp->ico  = new OIcon(); }
        tmp->hwnd = WinCreateWindow(client, WC_BUTTON, "", buttonFrameStyle,
                                    posx, posy, bWidth, bHeight,
                                    client, HWND_TOP, tmp->id, NULL, NULL); 
        break;

       case bitmaplike:
        tmp->hwnd = WinCreateWindow(client, WC_BUTTON, "", buttonFrameStyle,
                                     posx, posy, bWidth, bHeight,
                                     client, HWND_TOP, tmp->id, NULL, NULL);
         try {
           tmp->bmp = new OBitmap(tmp->hwnd, tmp->file); }
         catch (...) {
           delete tmp->bmp;
           tmp->bmp  = new OBitmap(); }
         hps = WinGetPS(tmp->hwnd); 
         break; }
    }
   WinShowWindow(tmp->hwnd, TRUE);
   tmp = buttons.getNext();
   i++;
  }
 showFrame();
}


void OButtonBar::setButtonDimensions(const ULONG height, const ULONG width)
{
 bHeight = height;
 bWidth = width; 
}

void OButtonBar::setResType(ULONG type) 
{
 resType = type; 
}


void OButtonBar::setPicSize(ULONG length)
{
 picSize = length;
}


void OButtonBar::setOrientation(ULONG type) 
{
 orientation = type; 
}


ULONG OButtonBar::getHeight() 
{
 return(bHeight); 
}


ULONG OButtonBar::getWidth() 
{
 return(bWidth); 
}



BOOL OButtonBar::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
   {
    case WM_PAINT:
    case WM_ERASEBACKGROUND:
      hps = WinBeginPaint(hwnd, 0, NULL);
      GpiCreateLogColorTable(hps, LCOL_RESET, LCOLF_RGB, 0L, 0L, NULL);
      WinQueryWindowRect(hwnd, &rcl);
      GpiSetColor(hps, pparms.Back);
      GpiBox(hps, DRO_FILL, (PPOINTL) &rcl.xRight, 0L, 0L);
      WinEndPaint(hps);
      break; 

    case WM_HITTEST:
      if ((!buttons.isEmpty()) && (bWidth != 0))
       {
        USHORT x;
     
        if (orientation == left2right)
          x = SHORT1FROMMP(mp1)/bWidth+1;
        else
          x = buttons.items - SHORT2FROMMP(mp1)/bHeight;

        if (x != lastHelpRequest) {
          pButtonInfo tmp = buttons.getItem(x);
 
          if (tmp) {
            WinSendMsg(parent, WM_CONTROL,
                              MPFROM2SHORT(OBUTTONBAR_MOVEOVER, tmp->id), tmp->helptext);
            lastHelpRequest = x; 
            return(TRUE); }
          else {
            WinSendMsg(parent, WM_CONTROL,
                              MPFROM2SHORT(OBUTTONBAR_MOVEOVER, 0), MPFROMP(""));
            lastHelpRequest = 0; }}
       }
      return(FALSE);

    case WM_BUTTON1DOWN:
    case WM_BUTTON2DOWN:
    case WM_BUTTON3DOWN: {
      pButtonInfo tmp = buttons.getItem(lastHelpRequest);

      if (tmp)
         WinSendMsg(parent, WM_CONTROL, MPFROM2SHORT(msg, tmp->id), mp1);
      break; }

    case WM_CONTROL:
      switch(SHORT2FROMMP(mp1)) {
        case BN_PAINT: {
          USHORT       x = SHORT1FROMMP(mp1);
          pButtonInfo  tmp = buttons.getFirst();
          PUSERBUTTON  pub;
          POINTL       ptl;

          while((tmp) && (tmp->id != x)) tmp = buttons.getNext();
          if (tmp)
             pub = (PUSERBUTTON) PVOIDFROMMP(mp2);
          else
             break;
          GpiCreateLogColorTable(pub->hps, LCOL_RESET, LCOLF_RGB, 0L, 0L, NULL);
          ptl.x = bWidth; ptl.y = bHeight;
          GpiSetColor(pub->hps, SYSCLR_BUTTONDARK);
          GpiBox(pub->hps, DRO_FILL, &ptl, 0L, 0L);

// draw button border
 
          GpiSetColor(pub->hps, SYSCLR_BUTTONMIDDLE);
          ptl.x = 2; ptl.y = 2; GpiMove(pub->hps, &ptl);
          if (pub->fsState == BDS_HILITED) {
             rcl.xLeft = (bWidth / 2) - (picSize/2 - 1);
             rcl.yBottom = (bHeight / 2) - (picSize/2 + 1);
             ptl.x = bWidth-2; ptl.y = bHeight-3;
             GpiBox(pub->hps, DRO_FILL, &ptl, 0L, 0L);
             GpiSetColor(pub->hps, SYSCLR_BUTTONLIGHT);
             ptl.x = 3; ptl.y = 2; GpiMove(pub->hps, &ptl);
             ptl.x = bWidth-2; ptl.y = 2; GpiLine(pub->hps, &ptl);
             ptl.x = bWidth-2; ptl.y = bHeight-3; GpiLine(pub->hps, &ptl); }
          else {
             rcl.xLeft = (bWidth - picSize) / 2;
             rcl.yBottom = (bHeight - picSize) /2 ;
             ptl.x = 2; ptl.y = 2; GpiMove(pub->hps, &ptl);
             ptl.x = bWidth-3; ptl.y = bHeight-2;
             GpiBox(pub->hps, DRO_FILL, &ptl, 0L, 0L);
             GpiSetColor(pub->hps, SYSCLR_BUTTONLIGHT);
             ptl.x = 2; ptl.y = bHeight-2; GpiLine(pub->hps, &ptl);
             ptl.x = bWidth-2; ptl.y = bHeight-2; GpiLine(pub->hps, &ptl); }


          if (resType == iconlike) {
             if (tmp->ico)
               WinDrawPointer(pub->hps, rcl.xLeft, rcl.yBottom, tmp->ico->hptr, DP_NORMAL); }
          else {
             if (tmp->bmp) {
               WinDrawBitmap(pub->hps, tmp->bmp->hbm, NULL, (PPOINTL)&rcl, 0, 0, DBM_NORMAL); }}
          break; }
         }
       break;

    case WM_COMMAND:
      WinSendMsg(parent, msg, mp1, mp2);
      break;

    case WM_CLOSE:
    case WM_DESTROY: {
      pButtonInfo tmp = buttons.getFirst();

      while(tmp) {
        if (tmp->ico) delete tmp->ico;
        if (tmp->bmp) delete tmp->bmp;
        tmp = buttons.getNext(); }
      break; }

    default:
      return(OFrame::OCommand(msg, mp1, mp2));
   }
 return(TRUE);
}




// end of source  
