// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OContainerBase.cpp


#define OINCL_OCONTAINER
#define OINCL_BASE

#include <ocl.hpp>



OContainerBase::OContainerBase(const ULONG id, const HWND Parent,
                               const ULONG cnrStyle, const ULONG initViewStyle)
  : OWindow(id, cnrStyle, 0)
{
 parent = owner = Parent;
 cnri.flWindowAttr = initViewStyle;
 hwnd = NULLHANDLE;
}


OContainerBase::OContainerBase(const pOFrame Parent, const ULONG id,
                               const ULONG cnrStyle, const ULONG initViewStyle)
  : OWindow(id, cnrStyle, 0)
{
 parent = owner = Parent->client;
 cnri.flWindowAttr = initViewStyle;
 hwnd = NULLHANDLE;
}


OContainerBase::OContainerBase(const OFrame& Parent, const ULONG id,
                               const ULONG cnrStyle, const ULONG initViewStyle)
  : OWindow(id, cnrStyle, 0)
{ 
 parent = owner = Parent.client;
 cnri.flWindowAttr = initViewStyle;
 hwnd = NULLHANDLE;
} 
 


OContainerBase::~OContainerBase() {}


BOOL OContainerBase::createContainer()
{
 if (!hwnd)
  {
   hwnd = WinCreateWindow(parent, WC_CONTAINER, NULL,
                          style | WS_VISIBLE, 0, 0, 0, 0,
                          parent, HWND_TOP, res, NULL, NULL);
   if (!hwnd)
     return(FALSE);
  }

 if (setupDetails() && setupItems()) {
   if (cnri.flWindowAttr & CA_CONTAINERTITLE)
     return(setTitle(winText.text));
   else
     return(setView(cnri.flWindowAttr)); }
 else
   return(FALSE);
}


BOOL OContainerBase::inherit(const HWND hwndDlg)
{
 return((OWindow::inherit(hwndDlg)) &&
        (createContainer()));
}


BOOL OContainerBase::freeDetails()
{
 if (!hwnd) return(FALSE);

 WinSendMsg(hwnd, CM_REMOVEDETAILFIELDINFO, NULL,
            MPFROM2SHORT(0, CMA_FREE | CMA_INVALIDATE));
 return(TRUE);
}


BOOL OContainerBase::freeItems()
{
 if (!hwnd) return(FALSE);

 WinSendMsg(hwnd, CM_REMOVERECORD, NULL,
            MPFROM2SHORT(0, CMA_FREE | CMA_INVALIDATE));
 return(TRUE);
}



BOOL OContainerBase::freeContainer()
{
 return(freeItems() && freeDetails());
}


BOOL OContainerBase::setTitle(PSZ title)
{
 if (title) {
   winText << title;
   cnri.pszCnrTitle = winText.text;
   return(setCnrInfo(CMA_FLWINDOWATTR | CMA_CNRTITLE) && invalidateDetails()); }
 return(FALSE);
}


BOOL OContainerBase::setView(ULONG viewStyle)
{
 cnri.flWindowAttr = viewStyle;
 return(setCnrInfo(CMA_FLWINDOWATTR) && invalidateDetails()); 
}


/***********************************************************************************************************/
/* Services                                                                                                */
/***********************************************************************************************************/


PFIELDINFO  OContainerBase::allocDetailFields(ULONG fields)
{
 return((PFIELDINFO)WinSendMsg(hwnd, 
                               CM_ALLOCDETAILFIELDINFO, 
                               MPFROMSHORT(fields), 
                               NULL));
}


PRECORDCORE OContainerBase::_allocRecords(ULONG additional, ULONG records)
{
 return((PRECORDCORE)WinSendMsg(hwnd, 
                                CM_ALLOCRECORD, 
                                MPFROMSHORT(additional), 
                                MPFROMSHORT(records)));
}


BOOL OContainerBase::arrangeRecords()
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_ARRANGE, 
                         NULL, 
                         NULL));
}



BOOL OContainerBase::collapseTree(PVOID rootRecord)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_COLLAPSETREE, 
                         MPFROMP(rootRecord), 
                         NULL));
}



BOOL OContainerBase::expandTree(PVOID rootRecord)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_EXPANDTREE, 
                         MPFROMP(rootRecord), 
                         NULL));
}



BOOL OContainerBase::openEdit(PCNREDITDATA editData)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_OPENEDIT, 
                         MPFROMP(editData), 
                         NULL));
}



BOOL OContainerBase::closeEdit()
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_CLOSEEDIT, 
                         NULL, 
                         NULL));
}



BOOL OContainerBase::eraseRecord(PVOID record)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_ERASERECORD, 
                         MPFROMP(record), 
                         NULL));
}


BOOL OContainerBase::freeDetailInfo(PFIELDINFO rootRecord, ULONG count)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_FREEDETAILFIELDINFO, 
                         MPFROMP(rootRecord), 
                         MPFROMSHORT(count)));
}


BOOL OContainerBase::freeRecord(PVOID rootRecord, ULONG count)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_FREERECORD, 
                         MPFROMP(rootRecord), 
                         MPFROMSHORT(count)));
}


BOOL OContainerBase::removeDetailInfo(PFIELDINFO rootRecord, ULONG count, ULONG flag)
{
 return((BOOL)WinSendMsg(hwnd,
                         CM_REMOVEDETAILFIELDINFO,
                         MPFROMP(rootRecord),
                         MPFROM2SHORT(count, flag)));
}


BOOL OContainerBase::removeRecord(PVOID rootRecord, ULONG count, ULONG flag)
{
 return((BOOL)WinSendMsg(hwnd,
                         CM_REMOVERECORD,
                         MPFROMP(rootRecord),
                         MPFROM2SHORT(count, flag)));
}


BOOL OContainerBase::filter(PFN function, PVOID storage)
{
 return(FALSE);
}


BOOL OContainerBase::insertDetailInfo(PFIELDINFO info, PFIELDINFOINSERT insert)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_INSERTDETAILFIELDINFO, 
                         MPFROMP(info), 
                         MPFROMP(insert)));
}


BOOL OContainerBase::insertRecord(PVOID rootRecord, PRECORDINSERT insert)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_INSERTRECORD, 
                         MPFROMP(rootRecord), 
                         MPFROMP(insert)));
}



BOOL OContainerBase::invalidateDetails()
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_INVALIDATEDETAILFIELDINFO, 
                         NULL, NULL));
}


BOOL OContainerBase::invalidateRecord(PVOID rootRecord, ULONG flag, ULONG count)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_INVALIDATERECORD, 
                         MPFROMP(rootRecord), 
                         MPFROM2SHORT(count, flag)));
}


ULONG OContainerBase::queryCnrInfo()
{
 return((ULONG)WinSendMsg(hwnd, 
                          CM_QUERYCNRINFO, 
                          MPFROMP(&cnri), 
                          MPFROMSHORT(sizeof(CNRINFO))));
}


PFIELDINFO OContainerBase::queryDetailInfo(PFIELDINFO rootRecord, ULONG flag)
{
 return((PFIELDINFO)WinSendMsg(hwnd, 
                               CM_QUERYDETAILFIELDINFO, 
                               MPFROMP(rootRecord),
                               MPFROMSHORT(flag)));
}


PRECORDCORE OContainerBase::_queryRecord(PVOID rootRecord, ULONG flag, ULONG order)
{
 return((PRECORDCORE)WinSendMsg(hwnd, 
                                CM_QUERYRECORD, 
                                MPFROMP(rootRecord), 
                                MPFROM2SHORT(flag, order)));
}


PRECORDCORE OContainerBase::_queryEmphasis(PVOID record, ULONG flag)
{
 return((PRECORDCORE)WinSendMsg(hwnd, 
                                CM_QUERYRECORDEMPHASIS, 
                                MPFROMP(record), 
                                MPFROMSHORT(flag)));
}



BOOL OContainerBase::setCnrInfo(ULONG attributeFlag)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_SETCNRINFO, 
                         MPFROMP(&cnri), 
                         MPFROMSHORT(attributeFlag)));
}



BOOL OContainerBase::setEmphasis(PVOID record, ULONG emphasis, BOOL set)
{
 return((BOOL)WinSendMsg(hwnd, 
                         CM_SETRECORDEMPHASIS, 
                         MPFROMP(record),
                         MPFROM2SHORT(emphasis, set)));
}


// end of source
