// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OEntryField.cpp


#define OINCL_BASE
#include <ocl.hpp>
#include <OEntryField.hpp>



OEntryField::OEntryField(const ULONG id,
                         const HWND Parent,
                         const ULONG Style,
                         const ULONG Limit)
  : OWindow(id, 0, 0),
    limit(Limit)
{
 parent = Parent;
 owner = Parent;
 style = Style;
}



OEntryField::OEntryField(const OFrame& Parent,
                         const ULONG Style,
                         const ULONG Limit)
  : OWindow(Parent.res, 0, 0),
    limit(Limit)
{
 parent = Parent.hwnd;
 owner = Parent.hwnd;
 style = Style;
}



OEntryField::OEntryField(const pOFrame Parent,
                         const ULONG Style,
                         const ULONG Limit)
  : OWindow(Parent->res, 0, 0),
    limit(Limit)
{
 parent = Parent->hwnd;
 owner = Parent->hwnd;
 style = Style;
}



OEntryField::~OEntryField() {}



BOOL OEntryField::createEntryField(const ULONG x, const ULONG y, 
                                   const ULONG cx, const ULONG cy)
{
 hwnd = WinCreateWindow(parent, WC_ENTRYFIELD, "", style | WS_VISIBLE,
                        x, y, cx, cy, owner, HWND_TOP, res, NULL, NULL);
 return((hwnd != NULLHANDLE) && (setTextLimit(limit)));
}



BOOL OEntryField::inherit(const HWND hwndDlg)
{
 return((OWindow::inherit(hwndDlg)) &&
        (setTextLimit(limit)));
}

BOOL  OEntryField::getText(OString& Buffer)
{
 PSZ   temp = new CHAR[limit];
 ULONG charsRead;

 if ((charsRead = (ULONG) WinQueryWindowText(hwnd, limit, temp)) != 0)
   Buffer << temp;
 else
   Buffer << (PSZ) NULL;
 delete[] temp;
 return(charsRead != 0);
}



BOOL  OEntryField::setText(PCSZ text)
{
 if (text)
   return(WinSetWindowText(hwnd, text));
 else
   return(WinSetWindowText(hwnd, ""));
}


BOOL OEntryField::setText(OString& Buffer)
{
 return(setText(Buffer.text));
}


BOOL  OEntryField::cutText()
{
 return((BOOL)WinSendMsg(hwnd, EM_CUT, NULL, NULL));
}


BOOL  OEntryField::copyText()
{
 return((BOOL)WinSendMsg(hwnd, EM_COPY, NULL, NULL));
}


BOOL  OEntryField::pasteText()
{
 return((BOOL)WinSendMsg(hwnd, EM_PASTE, NULL, NULL));
}


BOOL  OEntryField::clearText()
{
 return((BOOL)WinSendMsg(hwnd, EM_CLEAR, NULL, NULL));
}


BOOL  OEntryField::queryChanged()
{
 return((BOOL)WinSendMsg(hwnd, EM_QUERYCHANGED, NULL, NULL));
}


ULONG OEntryField::queryFirstChar()
{
 return((ULONG)WinSendMsg(hwnd, EM_QUERYFIRSTCHAR, NULL, NULL));
}



BOOL  OEntryField::queryRDOnly()
{
 return((BOOL)WinSendMsg(hwnd, EM_QUERYREADONLY, NULL, NULL));
}


BOOL  OEntryField::querySelection(USHORT& firstChar, USHORT& lastChar)
{
 MRESULT temp = WinSendMsg(hwnd, EM_QUERYSEL, NULL, NULL);

 return((temp != NULL) &&
        ((firstChar = SHORT1FROMMP(temp)) != 0) &&
        ((lastChar = SHORT2FROMMP(temp)) != 0));
}



BOOL  OEntryField::setFirstChar(const USHORT first)
{
 return((BOOL)WinSendMsg(hwnd, EM_SETFIRSTCHAR, MPFROMSHORT(first), NULL));
}


BOOL  OEntryField::setInsertMode(const BOOL insert)
{
 return((BOOL)WinSendMsg(hwnd, EM_SETINSERTMODE, MPFROMSHORT(insert), NULL));
}


BOOL  OEntryField::setRDOnly(const BOOL readOnly)
{
 return((BOOL)WinSendMsg(hwnd, EM_SETREADONLY, MPFROMSHORT(readOnly), NULL));
}


BOOL  OEntryField::setSelection(const USHORT firstChar, const USHORT lastChar)
{
 return((BOOL)WinSendMsg(hwnd, EM_SETSEL, MPFROM2SHORT(firstChar, lastChar), NULL));
}


BOOL  OEntryField::setTextLimit(const USHORT limit)
{
 return((BOOL)WinSendMsg(hwnd, EM_SETTEXTLIMIT, MPFROMSHORT(limit), NULL));
}


// end of source
