// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OFrame.cpp



#define OINCL_OFRAME

#include <ocl.hpp>


OFrame::OFrame(const ULONG id,
               const ULONG Style,
               const ULONG winbits)
 : OWindow(id, Style, winbits),
   frame(NULLHANDLE),
   client(NULLHANDLE)
 {}


OFrame::~OFrame()
{ 
 if (frame)
   WinDestroyWindow(frame); 
}


BOOL OFrame::createFrame(PCSZ className)
{
 FRAMECDATA fcdata;

 if (!OApp::currentOApp)
   return(FALSE);

 if (!WinRegisterClass(OApp::currentOApp->hab, (PSZ) className, OWinDefWindowProc, bits, sizeof(PVOID)))
   return(FALSE);

 fcdata.cb            = sizeof(FRAMECDATA);
 fcdata.flCreateFlags = style;
 fcdata.hmodResources = (HMODULE)NULL;
 fcdata.idResources   = res;

 frame = WinCreateWindow(parent, WC_FRAME,  "",
                                   0, 0, 0, 0, 0, owner, HWND_TOP, res, &fcdata, NULL);
 client = WinCreateWindow(frame, (PSZ) className, "", 0, 0, 0, 0, 0, frame,
                                  HWND_TOP, FID_CLIENT, this, NULL);

 if (winText.text) WinSetWindowText(frame, winText.text);

 return((frame!=NULLHANDLE) && (client!=NULLHANDLE));
}


void OFrame::centerFrame()
{
 SWP swpScreen;

 swpScreen.cx=WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN);
 swpScreen.cy=WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);

 WinSetWindowPos(frame, NULLHANDLE, (swpScreen.cx-sizepos.cx)/2,
                        (swpScreen.cy-sizepos.cy)/2, sizepos.cx, sizepos.cy,
                        SWP_SIZE | SWP_MOVE | SWP_RESTORE | SWP_ACTIVATE);
}


void OFrame::setSizePosFrame(const HWND behind, SWP dimensions, const ULONG flag)
{
 if (frame) {
   WinSetWindowPos(frame, behind, dimensions.x, dimensions.y, dimensions.cx, dimensions.cy, flag);
   WinQueryWindowPos(frame, &sizepos); }
}


void OFrame::showFrame() 
{
 if (frame)
    WinShowWindow(frame, TRUE);
}


void OFrame::hideFrame() 
{
 if (frame)
   WinShowWindow(frame, FALSE);
}


void OFrame::destroyFrame() 
{
 if (frame) {
    termFlag = TRUE;
    WinDestroyWindow(frame); }
}

void OFrame::postQuit()
{
 if (frame) {
    termFlag = TRUE;
    WinPostMsg(frame, WM_QUIT, NULL, NULL); }
}

void OFrame::setWindowText(PCSZ text) 
{
  if (text) {
     winText << (PSZ) text;
     WinSetWindowText(frame, winText.text); }
}



BOOL OFrame::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
   {
    case WM_PRESPARAMCHANGED:
      switch(LONGFROMMP(mp1))
       {
        case PP_FONTNAMESIZE:
          getFont();
          WinInvalidateRect(hwnd, 0, TRUE);
          WinQueryWindowPos(parent, &sizepos);
          WinSendMsg(parent, WM_SIZE, NULL, MPFROM2SHORT(sizepos.cx, sizepos.cy));
          break;

        case PP_FOREGROUNDCOLOR:
          getForeColor();
          WinInvalidateRect(hwnd, 0, TRUE);
          break;

        case PP_BACKGROUNDCOLOR:
          getBackColor();
          WinInvalidateRect(hwnd, 0, TRUE);
          break;
       }
      break;

    default:
      return(FALSE);
   }
 return(TRUE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif


// end of source  
