// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OList.c


#include <ocl.hpp>
#include <OListBase.hpp>


// OListBase

OListBase::OListBase() 
  : items(0), 
    actual(NULL), 
    first(NULL), 
    last(NULL) 
  {}



OListBase::~OListBase()
{
 init();
}



BOOL OListBase::isEmpty()
{
 return(first == NULL);
}


void OListBase::rewind()
{
 actual = first;
}


void OListBase::init()
{
 if (first)
  {
   PPVITEM tmp = last->prev;
   while(tmp)
    {
     delete tmp->next;
     tmp = tmp->prev;
    }
   delete first;
  }
 items = 0;
 actual = first = last = NULL;
}


void OListBase::reset()
{
 if (first)
  {
   PPVITEM tmp = last->prev;

   while(tmp)
    {
     freeItem(tmp->next->item);
     delete tmp->next;
     tmp = tmp->prev;
    }
   freeItem(first->item);
   delete first;
  }
 items = 0;
 actual = first = last = NULL;
}



void OListBase::addFirst(PVOID elem)
{
 PPVITEM aNew = new PVITEM;

 if (!elem) return;

 aNew->prev = NULL;
 aNew->next = first;
 aNew->item = elem;

 if (first)
   first->prev = aNew;
 else
   last = aNew;

 actual = first = aNew;
 items++;
}


void OListBase::addLast(PVOID elem)
{
 PPVITEM aNew = new PVITEM;

 if (!elem) return;

 if (!first)
  {
   aNew->prev = aNew->next = NULL;
   aNew->item = elem;
   first = actual = aNew;
  }
 else
  {
   aNew->prev = last;
   aNew->next = NULL;
   aNew->item = elem;
   last->next = aNew;
  }
 last = aNew;
 items++;
}



void OListBase::addAfter(PVOID previous, PVOID elem)
{
 PPVITEM tmp = first;

 if (!elem) return;

 if ((!previous) || (!first))
  {
   addFirst(elem);
   return;
  }

 while((tmp) && (tmp->item != previous))
   tmp = tmp->next;

 if (tmp)
   addAfter(tmp, elem);
 else
   addLast(elem);
}


// Element nach einem bestimmten ListItem einfgen

void OListBase::addAfter(PPVITEM previous, PVOID elem)
{
 PPVITEM aNew;

 if (!elem) return;

 if (!previous)
  {
   addFirst(elem);
   return;
  }

 if (!previous->next)
  {
   addLast(elem);
   return;
  }

 aNew = new PVITEM;

 aNew->next = previous->next;
 aNew->prev = previous;
 aNew->item = elem;

 previous->next->prev = aNew;
 previous->next = aNew;

 items++;
}



PVOID OListBase::_getFirst()
{
 if (first)
  {
   rewind();
   return(first->item);
  }
 return(NULL);
}



PVOID OListBase::_getLast()
{
 if (first)
  {
   actual = last;
   return(last->item);
  }
 return(NULL);
}



PVOID OListBase::_getNext()
{
 if (!first)
   return(NULL);
 else if (actual->next)
  {
   actual = actual->next;
   if (actual)
     return(actual->item);
   else
     return(NULL);
  }
 else
   return(NULL);
}




PVOID OListBase::_getPrev()
{
 if (!first)
   return(NULL);
 else if (actual->prev)
  {
   actual = actual->prev;
   if (actual)
     return(actual->item);
   else
     return(NULL);
  }
 else
   return(NULL);
}


PVOID OListBase::_getItem(ULONG itemNum)
{
 ULONG i;
 PVOID tmp;

 if ((!first) || (itemNum < 1))
   return(NULL);
 if (itemNum == 1)
   return(first->item);
 tmp = _getFirst();
 for (i=2; ((i <= itemNum) && (tmp)); i++)
   tmp = _getNext();
 return(tmp);
}


PVOID OListBase::_getNext(PVOID elem)
{
 PPVITEM tmp = first;

 if (!elem) return(NULL);

 while((tmp) && (tmp->item != elem))
   tmp = tmp->next;

 if (!tmp)
   return(NULL);
 else
   return(tmp->item);
}



PVOID OListBase::_getPrev(PVOID elem)
{
 PPVITEM tmp = last;

 if (!elem) return(NULL);

 while((tmp) && (tmp->item != elem))
    tmp = tmp->prev;

 if (!tmp)
   return(NULL);
 else
   return(tmp->item);
}



PPVITEM OListBase::_getPrevListItem(PVOID elem)
{
 PPVITEM tmp = last;

 if (!elem) return(NULL);

 while((tmp) && (tmp->item != elem))
   tmp = tmp->prev;

 return(tmp);
}


PPVITEM OListBase::_getNextListItem(PVOID elem)
{
 PPVITEM tmp = first;

 if (!elem) return(NULL);

 while((tmp) && (tmp->item != elem))
   tmp = tmp->next;

 return(tmp);
}


// Entfernen von ListItems aus der Liste, ohne die gespeicherten
// Daten zu lschen


void OListBase::remove(PVOID elem)
{
 PPVITEM tmp = first;

 if (!elem) return;

 while(tmp)
  {
   if ((tmp->item) && (tmp->item == elem))
    {
     if (tmp->prev)
       tmp->prev->next = tmp->next;
     else
       first = tmp->next;
     if (tmp->next)
       tmp->next->prev = tmp->prev;
     else
       last = tmp->prev;
     if (tmp == actual)
      {
       if (tmp->prev)
         actual = tmp->prev;
       else if (tmp->next)
         actual = tmp->next;
      }
     --items;
     tmp->item = NULL;
     if ((!tmp->prev) && (!tmp->next))
       init();
     else
      {
       delete tmp;
       tmp = NULL;
       return;
      }
    }
   tmp = tmp->next;
  }
}



void OListBase::del(PVOID elem)
{
 if ((first) && (elem))
  {
   remove(elem);
   freeItem(elem);
  }
}



// Ausgabe der kompletten Liste

void OListBase::print()
{
 PPVITEM tmp = first;

 while(tmp)
  {
   output(tmp->item);
   tmp = tmp->next;
  }
}



// end of source
