// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OListBox.cpp


#define OINCL_BASE
#include <ocl.hpp>
#include <OListBox.hpp>



OListBox::OListBox(const ULONG id,
                   const HWND Parent,
                   const ULONG Style)
  : OWindow(id, 0, 0)
{
 parent = Parent;
 owner = Parent;
 style = Style;
}



OListBox::OListBox(const OFrame& Parent,
                   const ULONG Style)
  : OWindow(Parent.res, 0, 0)
{
 parent = Parent.hwnd;
 owner = Parent.hwnd;
 style = Style;
}



OListBox::OListBox(const pOFrame Parent,
                   const ULONG Style)
  : OWindow(Parent->res, 0, 0)
{
 parent = Parent->hwnd;
 owner = Parent->hwnd;
 style = Style;
}



OListBox::~OListBox() {}



BOOL OListBox::createListBox(const ULONG x, const ULONG y,
                                   const ULONG cx, const ULONG cy)
{
 hwnd = WinCreateWindow(parent, WC_LISTBOX, "", style | WS_VISIBLE,
                        x, y, cx, cy, owner, HWND_TOP, res, NULL, NULL);
 return(hwnd != NULLHANDLE);
}



BOOL OListBox::deleteAll()
{
 return((BOOL)WinSendMsg(hwnd, LM_DELETEALL, NULL, NULL));
}



ULONG OListBox::deleteItem(const ULONG itemIndex)
{
 return((ULONG)WinSendMsg(hwnd, LM_DELETEITEM, MPFROMSHORT(itemIndex), NULL));
}



ULONG OListBox::insertItem(const PSZ text, const ULONG itemIndex)
{
 if (text)
   return((ULONG)WinSendMsg(hwnd, LM_INSERTITEM, MPFROMSHORT(itemIndex), MPFROMP(text)));
 return(0);
}


ULONG OListBox::insertItem(const OString& text, const ULONG itemIndex)
{
 return(insertItem(text.text, itemIndex));
}



ULONG OListBox::queryItemCount()
{
 return((ULONG)WinSendMsg(hwnd, LM_QUERYITEMCOUNT, NULL, NULL));
}


ULONG OListBox::queryItemHandle(const ULONG itemIndex)
{
 return((ULONG)WinSendMsg(hwnd, LM_QUERYITEMHANDLE, MPFROMSHORT(itemIndex), NULL));
}


ULONG OListBox::queryItemText(OString& Buffer, const ULONG itemIndex)
{
 PSZ   tmp = new CHAR[2*CCHMAXPATH];
 ULONG ul;

 ul = (ULONG) WinSendMsg(hwnd, LM_QUERYITEMTEXT, MPFROM2SHORT(itemIndex, 2*CCHMAXPATH), MPFROMP(tmp));
 Buffer << tmp;
 delete[] tmp;
 return(ul);
}


ULONG OListBox::queryItemTextLength(const ULONG itemIndex)
{
 return((ULONG)WinSendMsg(hwnd, LM_QUERYITEMTEXTLENGTH, MPFROMSHORT(itemIndex), NULL));
}


ULONG OListBox::querySelection(const ULONG itemStart)
{
 return((ULONG)WinSendMsg(hwnd, LM_QUERYSELECTION, MPFROMSHORT(itemStart), NULL));
}


ULONG OListBox::queryTopItem()
{
 return((ULONG)WinSendMsg(hwnd, LM_QUERYTOPINDEX, NULL, NULL));
}



ULONG OListBox::searchString(const PSZ text, 
                             const ULONG itemStart,
                             const ULONG command)
{
 if (text)
   return((ULONG)WinSendMsg(hwnd, LM_SEARCHSTRING, MPFROM2SHORT(command, itemStart), MPFROMP(text)));
 return(0);
}



ULONG OListBox::searchString(const OString& text,
                             const ULONG itemStart,
                             const ULONG command)
{
 return(searchString(text.text, itemStart, command));
}


BOOL  OListBox::selectItem(const ULONG itemIndex, const BOOL state)
{
 return((BOOL)WinSendMsg(hwnd, LM_SELECTITEM, MPFROMSHORT(itemIndex), MPFROMSHORT(state)));
}


BOOL  OListBox::setItemHandle(const ULONG itemIndex, const ULONG handle)
{
 return((BOOL)WinSendMsg(hwnd, LM_SETITEMHANDLE, MPFROMSHORT(itemIndex), MPFROMSHORT(handle)));
}


BOOL  OListBox::setItemHeight(const ULONG newHeight)
{
 return((BOOL)WinSendMsg(hwnd, LM_SETITEMHEIGHT, MPFROMSHORT(newHeight), NULL));
}


BOOL OListBox::setItemText(const ULONG itemIndex, const PSZ text)
{
 if (text)
   return((BOOL)WinSendMsg(hwnd, LM_SETITEMTEXT, MPFROMSHORT(itemIndex), MPFROMP(text)));
 return(FALSE);
}


BOOL OListBox::setItemText(const ULONG itemIndex, const OString& text)
{
 return(setItemText(itemIndex, text.text));
}


BOOL  OListBox::setToTop(const ULONG itemIndex)
{
 return((BOOL)WinSendMsg(hwnd, LM_SETTOPINDEX, MPFROMSHORT(itemIndex), NULL));
}


// end of source
