// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OMLE.cpp


#define OINCL_OMLE

#include <ocl.hpp>


OMLE::OMLE(const ULONG id,
           const HWND Parent,
           const ULONG Style)
     : OWindow(id, 0, 0),
       imex(NULL),
       imexLen(0),
       imexOffset(0),
       insertLargeThr(this, &OMLE::insertLarge)
{
 parent = owner = Parent;
 style = Style;
}


OMLE::OMLE(const OFrame& Parent,
           const ULONG Style)
     : OWindow(Parent.res, 0, 0),
       imex(NULL),
       imexLen(0),
       imexOffset(0),
       insertLargeThr(this, &OMLE::insertLarge)
{
 parent = owner = Parent.hwnd;
 style = Style;
}

OMLE::OMLE(const pOFrame Parent,
           const ULONG Style)
     : OWindow(Parent->res, 0, 0),
       imex(NULL),
       imexLen(0),
       imexOffset(0),
       insertLargeThr(this, &OMLE::insertLarge)
{
 parent = owner = Parent->hwnd;
 style = Style;
}



OMLE::~OMLE() {}


BOOL OMLE::createMLE(const ULONG x,
                     const ULONG y,
                     const ULONG cx,
                     const ULONG cy)
{
 hwnd = WinCreateWindow(parent, WC_MLE, NULL, style | WS_VISIBLE,
                        x, y, cx, cy,
                        owner, HWND_TOP, 0, NULL, NULL);
 return(hwnd != NULLHANDLE);
}



ULONG OMLE::getText(PVOID out, ULONG len)
{
 ULONG  exported;

 WinSendMsg(hwnd, MLM_SETIMPORTEXPORT, MPFROMP((PBYTE)out), MPFROMLONG(len));
 exported = (ULONG) WinSendMsg(hwnd, MLM_EXPORT, MPFROMP(&imexOffset), MPFROMP(&len));
 return(exported);
}


ULONG OMLE::getText(OString& Buffer)
{
 PSZ   temp = new CHAR[3*CCHMAXPATH];
 ULONG charsRead;

 if ((charsRead = (ULONG) WinQueryWindowText(hwnd, 3*CCHMAXPATH, temp)) != 0)
   Buffer << temp;
 else
   Buffer << (PSZ) NULL;
 delete[] temp;
 return(charsRead != 0);
}



ULONG OMLE::setText(PVOID in, ULONG len)
{
 ULONG  inserted = imexOffset;

 WinSendMsg(hwnd, MLM_SETIMPORTEXPORT, MPFROMP((PBYTE)in), MPFROMLONG(len));
 WinSendMsg(hwnd, MLM_IMPORT, MPFROMP(&imexOffset), MPFROMP(&len));

 inserted = imexOffset - inserted;
 return(inserted);
}


ULONG OMLE::setText(PSZ text)
{
 if (text)
   return(setText((PVOID)text, strlen(text)+1));
 return(0);
}


ULONG OMLE::setText(OString& text)
{
 return(setText(text.text));
}


BOOL OMLE::insertText(PSZ text)
{
 if (text)
   return((BOOL)WinSendMsg(hwnd, MLM_INSERT, MPFROMP(text), NULL));
 return(FALSE);
}


BOOL OMLE::insertText(OString& text)
{
 return(insertText(text.text));
}



BOOL OMLE::insertFile(PCSZ fileName)
{
 openedFile << (PSZ) fileName;
 insertLargeThr.run();
 return(TRUE);
}



BOOL OMLE::copy()
{
 return(FALSE);
}


BOOL OMLE::cut()
{
 return(FALSE);
}


BOOL OMLE::paste()
{
 return(FALSE);
}


BOOL OMLE::copy2Clip()
{
 return(FALSE);
}


BOOL OMLE::cut2Clip()
{
 return(FALSE);
}


BOOL OMLE::pasteFromClip()
{
 return(FALSE);
}


BOOL OMLE::clear()
{
 return(TRUE);
}




BOOL OMLE::undo()
{
 return(FALSE);
}




BOOL OMLE::deleteText(ULONG beginPoint, ULONG count)
{
 return(((ULONG)WinSendMsg(hwnd, MLM_DELETE,
                           MPFROMSHORT(beginPoint), MPFROMSHORT(count))) == count);
}

BOOL  OMLE::deleteAll()
{
 return(((ULONG)WinSendMsg(hwnd, MLM_DELETE, 
                           MPFROMSHORT(0), MPFROMSHORT(length()))) == length());
}


void OMLE::insertLarge()
{
 FILESTATUS fileStatus;
 PBYTE      rBuffer;
 HFILE      hf;
 ULONG      cbRead, ulAction;
 PBYTE      Buffer;
 ULONG      SIZE = 8192;

 DosOpen(openedFile.text, &hf, &ulAction, 0, FILE_NORMAL, FILE_OPEN | FILE_CREATE,
         OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE, NULL);
 DosQueryFileInfo(hf, 1, (PVOID)&fileStatus,  sizeof(FILESTATUS));
 DosAllocMem((PPVOID)&rBuffer, fileStatus.cbFileAlloc, PAG_READ | PAG_WRITE | PAG_COMMIT);
 memset(rBuffer, 0, fileStatus.cbFileAlloc);

 DosRead(hf, rBuffer, fileStatus.cbFileAlloc, &cbRead);
 DosClose(hf);

 if (!cbRead)
   return;

 if (cbRead > SIZE) {
   ULONG steps = cbRead / SIZE;
   ULONG rest = cbRead % SIZE;

   Buffer = rBuffer;
   for (ULONG i = 1; i <= steps; ++i) {
     setText(Buffer, SIZE);
     Buffer = Buffer+SIZE; }
   setText(Buffer, rest); }
 else
   setText(rBuffer, fileStatus.cbFileAlloc);

 DosFreeMem(rBuffer);
}



ULONG OMLE::length()
{
 return((ULONG)WinSendMsg(hwnd, MLM_QUERYFORMATTEXTLENGTH, MPFROMLONG(0), MPFROMLONG(-1)));
}


ULONG OMLE::lines()
{
 return((ULONG)WinSendMsg(hwnd, MLM_QUERYLINECOUNT, NULL, NULL));
}



BOOL OMLE::changed()
{
 return((BOOL)WinSendMsg(hwnd, MLM_QUERYCHANGED, NULL, NULL));
}


void OMLE::enableRefresh()
{
 WinSendMsg(hwnd, MLM_ENABLEREFRESH, NULL, NULL);
}


void OMLE::disableRefresh()
{
 WinSendMsg(hwnd, MLM_DISABLEREFRESH, NULL, NULL);
}


ULONG OMLE::setFormat(const ULONG format)
{
 return((ULONG)WinSendMsg(hwnd, MLM_FORMAT, MPFROMSHORT(format), NULL));
}



// end of source
