// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OMuxSem.cpp


#define INCL_DOS
#define INCL_ERRORS
#define INCL_DOSERRORS
#define OINCL_OMUXSEM

#include <ocl.hpp>

PSZ OMuxERROR = "OMuxSem Error";


OMuxSem::OMuxSem(HMTX handle)
  : hmtx(handle),
    muxState(0)
{
 if (!openMuxSem(handle))
  throw OVioException(OMuxERROR, muxState);
}

OMuxSem::OMuxSem(PCSZ name, ULONG action, BOOL shared)
{
 APIRET rc;

 switch(action)
  {
   case OMuxSem::create: {
    ULONG  shareFlag = 0;

    if ((!name) && (shared))
      shareFlag = DC_SEM_SHARED;
    if((rc = DosCreateMutexSem(name, &hmtx, shareFlag, FALSE))!=0)
      throw OVioException(OMuxERROR, rc);
    break; }

   case OMuxSem::open:
    if (!openMuxSem(name))
      throw OVioException(OMuxERROR, muxState);
    break;
  }
}


OMuxSem::~OMuxSem()
{
 if (hmtx)
   closeMuxSem();
}


BOOL OMuxSem::openMuxSem(HMTX handle)
{
 hmtx = handle;
 muxState = DosOpenMutexSem((PSZ)NULL, &hmtx);
 return(muxState == 0);
}


BOOL OMuxSem::openMuxSem(PCSZ name)
{
 muxName << (PSZ) name;
 muxState = DosOpenMutexSem(muxName.text, &hmtx);
 return(muxState == 0);
}


BOOL OMuxSem::closeMuxSem()
{
 muxState = DosCloseMutexSem(hmtx);
 hmtx = (HMUX)NULL;
 return(muxState == 0);
}


BOOL OMuxSem::requestMuxSem(ULONG timeOut)
{
 muxState = DosRequestMutexSem(hmtx, timeOut);
 return(muxState == 0);
}


BOOL OMuxSem::releaseMuxSem()
{
 muxState = DosReleaseMutexSem(hmtx);
 return(muxState == 0);
}


void OMuxSem::setMuxSemHandle(HMTX handle)
{
 hmtx = handle;
}


HMTX OMuxSem::getMuxSemHandle()
{
 return(hmtx);
}


// end of source
