// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OMuxWaitSem.cpp


#define INCL_DOS
#define OINCL_OMUXWAITSEM

#include <ocl.hpp>


PSZ  OMuxWaitERROR = "OMuxWaitSem Error";


OMuxWaitSem::OMuxWaitSem(HMUX handle)
  : hmux((HMUX)NULL),
    muxwaitState(0)
{
 if (!openMuxWaitSem(handle))
   throw OVioException(OMuxWaitERROR, muxwaitState);
}


OMuxWaitSem::OMuxWaitSem(PCSZ name, 
                         ULONG numSem, 
                         PSEMRECORD semArray,  
                         ULONG action,
                         ULONG attributes)
  : hmux((HMUX)NULL),
    muxwaitState(0)
{
 switch(action)
  {
   case OMuxWaitSem::create: {
    APIRET rc;

    if ((rc=DosCreateMuxWaitSem(name, &hmux, numSem, semArray, attributes))!=0)
      throw OVioException(OMuxWaitERROR, rc);
    break; }

   case OMuxWaitSem::open:
    if (!openMuxWaitSem(name))
      throw OVioException(OMuxWaitERROR, muxwaitState);
    break;
  }
}


OMuxWaitSem::~OMuxWaitSem()
{
 if (hmux)
   closeMuxWaitSem();
}



ULONG OMuxWaitSem::waitMuxWaitSem(ULONG timeout)
{
 ULONG posted = 0;

 muxwaitState = DosWaitMuxWaitSem(hmux, timeout, &posted);
 if (muxwaitState)
   return(0);
 return(posted);
}



BOOL OMuxWaitSem::openMuxWaitSem(HMUX handle)
{
 muxwaitState = DosOpenMuxWaitSem((PSZ)NULL, &handle);
 return(muxwaitState == 0);
}


BOOL OMuxWaitSem::openMuxWaitSem(PCSZ name)
{
 muxwaitState = DosOpenMuxWaitSem((PSZ)name, &hmux);
 return(muxwaitState == 0);
}



BOOL OMuxWaitSem::addSemaphore(PSEMRECORD record)
{
 muxwaitState = DosAddMuxWaitSem(hmux, record);
 return(muxwaitState == 0);
}


BOOL OMuxWaitSem::addSemaphore(ONSem *sema)
{
 SEMRECORD semRec;

 if (!sema) {
   muxwaitState = ERROR_INVALID_PARAMETER;
   return(FALSE); }

 semRec.hsemCur = (HSEM) sema->getSemHandle();
 return(addSemaphore(&semRec));
}


BOOL OMuxWaitSem::addSemaphore(OMuxSem *sema)
{
 SEMRECORD semRec;

 if (!sema) {
   muxwaitState = ERROR_INVALID_PARAMETER;
   return(FALSE); }

 semRec.hsemCur = (HSEM) sema->getMuxSemHandle();
 return(addSemaphore(&semRec));
}


BOOL OMuxWaitSem::delSemaphore(HSEM handle)
{
 muxwaitState = DosDeleteMuxWaitSem(hmux, handle);
 return(muxwaitState == 0);
}


BOOL OMuxWaitSem::queryMuxWaitSem(PULONG num,
                                  PSEMRECORD semArray,
                                  PULONG attributes)
{
 muxwaitState = DosQueryMuxWaitSem(hmux, num, semArray, attributes);
 return(muxwaitState == 0);
}


void OMuxWaitSem::setMuxWaitSemHandle(HMUX handle)
{
 hmux = handle;
}


HMUX OMuxWaitSem::getMuxWaitSemHandle()
{
 return(hmux);
}


BOOL OMuxWaitSem::closeMuxWaitSem()
{
 muxwaitState = DosCloseMuxWaitSem(hmux);
 return(muxwaitState == 0);
}


// end of source
