// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// ONSem.cpp
// Named Semaphores

// class member functions


#define INCL_DOS
#define INCL_ERRORS
#define INCL_DOSERRORS
#define OINCL_ONSEM

#include <ocl.hpp>
#include <bseerr.h>


PSZ semERR = "ONSem Error";


ONSem::ONSem(HEV handle)
  : sema(handle),
    semState(0)
{
 if (!openSem(sema))
   throw OVioException(semERR, semState);
}


ONSem::ONSem(PCSZ name, ULONG action, BOOL initialState)
  : sema((HEV)NULL), 
    semState(0),
    semName((PSZ)name)
{
 switch(action)
  {
   case ONSem::create: {
    APIRET rc;
    if((rc = DosCreateEventSem(semName.text, &sema, 0, initialState)) != 0)
      throw OVioException(semERR, rc);
    break; }

   case ONSem::open:
    if (!openSem(name))
      throw OVioException(semERR, semState);
    if (initialState)
      postSem();
    break;
  }
}



ONSem::~ONSem()
{
 if (sema) closeSem();
}


void ONSem::setSemHandle(HEV hev) 
{
 sema = hev;
}


HEV  ONSem::getSemHandle()
{ 
 return(sema); 
}


BOOL ONSem::waitSem(const ULONG timeout)
{
 semState = DosWaitEventSem(sema, timeout);
 return(semState == 0);
}



BOOL ONSem::postSem()
{
 semState = DosPostEventSem(sema);
 return(semState == 0);
}


BOOL ONSem::closeSem()
{
 semState = DosCloseEventSem(sema);
 sema = (HEV)NULL;
 return(semState == 0);
}


BOOL ONSem::openSem(PCSZ name)
{
 
 if (!name) {
   semState = ERROR_INVALID_NAME;
   return(FALSE); }
 else {
   sema = (HEV) NULL;
   semName << (PSZ) name;
   semState = DosOpenEventSem(semName.text, &sema); }
 return(semState == 0);
}


BOOL ONSem::openSem(const HEV aSem)
{
 sema = aSem;
 semName << (PSZ) NULL;
 semState = DosOpenEventSem(semName.text, &sema);
 return(semState == 0);
}


ULONG ONSem::resetSem()
{
 ULONG  postCount = 0;

 semState = DosResetEventSem(sema, &postCount);
 return(postCount);
}


ULONG ONSem::querySem()
{
 ULONG  postCount = 0;

 semState = DosQueryEventSem(sema, &postCount);
 return(postCount);
}


// end of source
