// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OPipeCli.cpp


#define INCL_DOS
#define OINCL_ONPIPE;
#define OINCL_BASE

#include <ocl.hpp>
#include <bseerr.h>


// members of OPipeCli


OPipeCli::OPipeCli(const ULONG Size,
                   const ULONG openMode,
                   const ULONG pipeMode,
                   const ULONG timeout,
                   const BOOL  waitFor)
  : pipeThread(this, &OPipeCli::Pipe_IO)
{
 oMode = openMode;
 pMode = pipeMode;
 timeOut = timeout;
 BufSize = Size;
 waitForInstance = waitFor;
 connected = FALSE;
 Pipe = (HPIPE)NULL;
}



OPipeCli::~OPipeCli()
{
 if (connected) disconnectPipe();
}


BOOL OPipeCli::beginPiping(PCSZ pipeName) 
{
 return(connectPipe(pipeName)); 
}


BOOL OPipeCli::stopPiping() 
{
 return(disconnectPipe()); 
}


void OPipeCli::setSize(const ULONG Size) 
{
 BufSize = Size; 
}


BOOL OPipeCli::connectPipe(PCSZ pipeName)
{
 ULONG     Action;
 APIRET    rc;

 if ((!oMode) || (connected) || (!Buffer) || (!BufSize))
   return(FALSE);

 while(!close)
  {
   rc = DosOpen((PSZ)pipeName, (PHFILE)&Pipe, &Action, 0L, 0L, 
                OPEN_ACTION_OPEN_IF_EXISTS, oMode, NULL);
   switch(rc)
    {
     case NO_ERROR:
       connected = TRUE;
       return(pipeThread.run());

     case ERROR_PIPE_BUSY:
       if (!waitForInstance)
         return(FALSE);
       DosWaitNPipe((PSZ)pipeName, 0);
       break;
    }
  }
 return(FALSE);
}

BOOL OPipeCli::disconnectPipe()
{
 close = TRUE;
 DosClose(Pipe);
 connected = FALSE;
 if (pipeThread.isRunning())
   pipeThread.stop();
 return(TRUE);
}

BOOL OPipeCli::postPipe(PVOID pvData, ULONG client)
{
 ULONG written = 0;

 return((connected) && 
        (!DosWrite(Pipe, pvData, BufSize, &written)) &&
        (written != 0));
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif


void OPipeCli::Pipe_IO()
{
 ULONG read = 0;

 while (!close)
   {
    if (!DosRead(Pipe, Buffer, BufSize, &read) && read > 0)
      OPipeCommand(Buffer); 
    else
      DosSleep(100); 
   }

 connected = FALSE;
 _endthread();
}

// end of source
