// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OPipeSvr.cpp

// members of OPipeSvr

#define INCL_DOS
#define OINCL_ONPIPE;
#define OINCL_BASE

#include <ocl.hpp>
#include <bseerr.h>


OPipeSvr::OPipeSvr(const ULONG inSize,
                   const ULONG outSize,
                   const ULONG Size,
                   const ULONG openMode,
                   const ULONG pipeMode,
                   const ULONG timeout,
                   const ULONG inst)
  : pipeThread(this, &OPipeSvr::Pipe_IO)
{
 oMode = openMode;
 pMode = pipeMode;
 timeOut = timeout;
 instances = inst;
 inBuf = inSize;
 outBuf = outSize;
 BufSize = Size;
}


OPipeSvr::~OPipeSvr()
{
 destroyPipe();
 pipes.reset();
}


BOOL OPipeSvr::beginPiping(PCSZ pipeName)
{
 return(createPipe(pipeName)); 
}


BOOL OPipeSvr::stopPiping() 
{
 return(destroyPipe()); 
}


void OPipeSvr::setBufSizes(const ULONG in, const ULONG out, const ULONG Size)
{
 inBuf = in;
 outBuf = out;
 BufSize = Size;
}


BOOL OPipeSvr::createPipe(PCSZ pipeName)
{
 if ((!oMode) || (!pMode) || (!timeOut) || (!inBuf) || (!outBuf) || (!Buffer) || (!BufSize))
   return(FALSE);

 pMode |= instances;
 connectedClients = 0;

 for (ULONG i=0; i < instances; i++) {
   pPipeStatus status = new PipeStatus;

   status->connected = FALSE;
   if (DosCreateNPipe((PSZ) pipeName,  &status->pipe,  oMode, pMode, outBuf, inBuf, timeOut))
     delete status;
   else
     pipes.addLast(status);
  }

 if (pipes.items == 0)
   return(FALSE);

 return(pipeThread.run());
}


BOOL OPipeSvr::destroyPipe()
{
 close = TRUE;
 pipeThread.waitFor();
 actual = pipes.getFirst();
 while(actual) {
   actual->connected = FALSE;
   DosDisConnectNPipe(actual->pipe);
   DosClose(actual->pipe); 
   actual = pipes.getNext(); }
 return(TRUE);
}


BOOL OPipeSvr::getPipeData()
{
 ULONG read = 0;
 if ((!actual->connected) || (DosRead(actual->pipe, Buffer, BufSize, &read)))
   return(FALSE);
 if (read == 0)
   return(FALSE);
 OPipeCommand(Buffer);
 return(TRUE);
}

BOOL OPipeSvr::postPipe(PVOID pvData, ULONG client)
{
 ULONG                written = 0;
 pPipeStatus          postTo = NULL;
 ListItem<PipeStatus> *tmp = (ListItem<PipeStatus>*)pipes.last;

 if (client == 0)
  {
   BOOL fSuccess = TRUE;

   for(ULONG client2post = 1; 
      ((client2post <= connectedClients) && (fSuccess));
      client2post++)
     fSuccess = postPipe(pvData, client2post);
   return(fSuccess);
  }

 if ((pipes.isEmpty()) || (client < 1))
   return(FALSE);
 if (client > 1)
  {
   for (ULONG i=1; i < client; i++)
    {
     tmp = tmp->prev;
     if (!tmp)
       return(FALSE);
    }
  }
 
 if (tmp)
   postTo = tmp->item;

 return((postTo != NULL) &&
        (postTo->connected) && 
        (!DosWrite(postTo->pipe, pvData, BufSize, &written)) && 
        (written != 0)); 
}


void OPipeSvr::Pipe_IO()
{
 APIRET     rc;
 
 pipeThread.setPrty(PRTYC_IDLETIME, 0);  // initially run at idle priority

 actual = pipes.getFirst();

 while((!close) && (actual))
  {
   rc = DosConnectNPipe(actual->pipe);
   switch(rc) 
    {
     case NO_ERROR:
       if (actual->connected == FALSE) {
           actual->connected = TRUE;
           connectedClients++; }
       if (connectedClients > 0)
         pipeThread.setPrty(PRTYC_REGULAR, 0);
       getPipeData();
       break;
 
     case ERROR_BROKEN_PIPE:
       actual->connected = FALSE;
       connectedClients--;
       DosDisConnectNPipe(actual->pipe);
       if (connectedClients == 0)
         pipeThread.setPrty(PRTYC_IDLETIME, 0);
       break;

     case ERROR_PIPE_NOT_CONNECTED:
       actual->connected = FALSE;
       break;
    }
   actual = pipes.getNext();
   if (!actual) {
      actual = pipes.getFirst();
      DosSleep(100); }
  }

 _endthread();
}

// end of source
