// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OPMException.cpp


#define OINCL_OPMEXCEPTION
#include <ocl.hpp>


OPMException::OPMException(const PSZ err,
                           const ULONG retcode,
                           const ULONG severity,
                           const PSZ source,
                           const ULONG line)
  : OException(err, retcode, severity, source, line),
    errRes(0)
  {}


OPMException::OPMException(const OString& err,
                           const ULONG retcode,
                           const ULONG severity,
                           const PSZ source,
                           const ULONG line)
  : OException(err, retcode, severity, source, line),
    errRes(0)
  {}


OPMException::OPMException(const ULONG errResource, 
                           const ULONG retcode,     
                           const ULONG severity,
                           const HMODULE mod,
                           const PSZ source,
                           const ULONG line)        
  : OException(NULL, retcode, severity, source, line),
    OMsgs(mod),
    errRes(errResource)
  {}


OPMException::OPMException(const OPMException& ex)
  : OException(ex.description.text, ex.rc, ex.state, ex.Source.text, ex.Line),
    OMsgs(ex.handle),
    message(ex.message.text),
    errRes(ex.errRes)
  {
   if (errRes) {
     loadString(errRes, 2*CCHMAXPATH, Msg);
     description << Msg; }
  }


OPMException::~OPMException() {}



void OPMException::viewError()
{
 if (OApp::currentOApp)
  {
   PERRINFO  pErrInfoBlk;
   PSZ       pszOffset;
   CHAR      ErrBuffer[512];
   ULONG     lastErr = WinGetLastError(OApp::currentOApp->hab);

   sprintf(ErrBuffer,
           "An Exception was thrown in Module: %s\nLinenumber in Sourcecode: %d\n"\
           "RetCode: %d (0x%X)\nWinErr: %d (0x%X)\nError reported:\n",
           Source.text, Line, rc, rc, lastErr, lastErr);

   while((pErrInfoBlk = WinGetErrorInfo(OApp::currentOApp->hab)) != (PERRINFO)NULL) {
     pszOffset = ((PSZ)pErrInfoBlk) + pErrInfoBlk->offaoffszMsg;
     strcat(ErrBuffer, (((PSZ)pErrInfoBlk) + *((PSHORT)pszOffset)));
     WinFreeErrorInfo(pErrInfoBlk); }

   message << ErrBuffer;

   if (errRes) {
     loadString(errRes, 2*CCHMAXPATH, Msg);
     message + Msg; }
   else
     message + description;
  }

 error("Note!", message.text);
}


// end of source
