// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OProfile.cpp

// Profile Actions (INI-Handling)

#define OINCL_OPROFILE
#define OINCL_OAPP
#define OINCL_BASE

#define INCL_WIN
#define INCL_PM

#include <ocl.hpp>


// members of OProfile


OProfile::OProfile() : hini(NULLHANDLE) {}


OProfile::OProfile(const HINI iniHandle) : hini(iniHandle) {}


OProfile::OProfile(PSZ iniName)
{
 hini = PrfOpenProfile(OApp::currentOApp->hab, iniName);
}


BOOL OProfile::open(PSZ iniName)
{
 hini = PrfOpenProfile(OApp::currentOApp->hab, (PSZ) iniName);
 return(hini != NULLHANDLE);
}


OProfile::~OProfile()
{
 list.reset();
 PrfCloseProfile(hini);
}


void OProfile::close()
{
 PrfCloseProfile(hini);
}


BOOL OProfile::write(PSZ App, PSZ Key, PVOID Value, ULONG len)
{
 return(PrfWriteProfileData(hini, (PSZ) App, (PSZ) Key, Value, len));
}


BOOL OProfile::deleteApp(PSZ App)
{
 return(PrfWriteProfileData(hini, App, NULL, NULL, 0));
}


BOOL OProfile::deleteKey(PSZ App, PSZ Key)
{
 return(PrfWriteProfileData(hini, App, Key, NULL, 0));
}

BOOL OProfile::getApps()
{
 return(getKeys(NULL));
}


BOOL OProfile::getKeys(PSZ App)
{
 UCHAR  *DatTemp;
 ULONG  Size = 0x0000ffffL;

 if (DosAllocMem((PPVOID) &DatTemp,  0x0000ffffL, PAG_READ | PAG_WRITE | PAG_COMMIT))
   return(FALSE);

 if (getData(App, NULL, DatTemp, &Size))
  return(setupList(DatTemp));
 else
  return(FALSE);
}


BOOL OProfile::getStr(PSZ App, PSZ Key)
{
 UCHAR  *DatTemp;
 ULONG  Size = 0x0000ffffL;

 if (DosAllocMem((PPVOID) &DatTemp,  0x0000ffffL, PAG_READ | PAG_WRITE | PAG_COMMIT))
   return(FALSE);

 if (!getData(App, Key, DatTemp, &Size))
   return(FALSE);
 else
   val << (PSZ) DatTemp;

 DosFreeMem(DatTemp);
 return(TRUE);
}


BOOL OProfile::getData(PSZ App, PSZ Key, PVOID Buffer, PULONG Size)
{
 return(PrfQueryProfileData(hini, App, Key, Buffer, Size));
}


BOOL OProfile::setupList(UCHAR *DatTemp)
{
 ULONG x, len;
 UCHAR *Buffer;

 if (DosAllocMem((PPVOID) &Buffer,  0x0000ffffL, PAG_READ | PAG_WRITE | PAG_COMMIT))
   return(FALSE);

 list.reset();

 x = 0;
 do
   {
    len = 0;
    while (DatTemp[x+len] != '\0')
      {
       Buffer[len] = DatTemp[x+len];
       ++len;
      }
    Buffer[len] = '\0';
    if (len != 0)
      {
       list << (PSZ) Buffer;
       x += (len+1);
      }
   }
   while (DatTemp[x] != '\0');

 DosFreeMem(Buffer);
 DosFreeMem(DatTemp);
 return(TRUE);
}


BOOL OProfile::insert(const HWND hwnd, const ULONG id)
{
 ListItem<OString> *tmp = (ListItem<OString>*) list.first;

 while((tmp) && (tmp->item) && (tmp->item->text))
  {
   if (LIT_MEMERROR == (LONG)WinSendDlgItemMsg(hwnd, id, LM_INSERTITEM,
                            MPFROM2SHORT(LIT_SORTASCENDING, 0),
                            MPFROMP(tmp->item->text)))
     return(FALSE);
   tmp = tmp->next;
  }

 tmp = (ListItem<OString>*) list.first;
 if ((tmp) && (tmp->item) && (tmp->item->text))
    WinSetDlgItemText(hwnd, id, tmp->item->text);
 return(TRUE);
}


// end of source
