// OCL - Class Library for OS/2 2.x and up
// (c) 1994 Matthias Stbner
// All Rights Reserved
// OSlider.cpp


#include <OCL.HPP>
#include "OSlider.hpp"


OSlider::OSlider(const ULONG ID,
                 const HWND  Parent,  // becomes parent and owner of the slider
                 const ULONG scale1,
                 const ULONG space1,
                 const ULONG scale2,
                 const ULONG space2,
                 const ULONG Style)
  : OWindow(ID, 0, 0),
    Scale1(scale1),
    Space1(space1),
    Scale2(scale2),
    Space2(space2)
{
 parent = Parent;
 owner  = Parent;
 style  = Style;
}


OSlider::OSlider(const OFrame& Parent,
                 const ULONG scale1,
                 const ULONG space1,
                 const ULONG scale2,
                 const ULONG space2,
                 const ULONG Style)
  : OWindow(Parent.res, 0, 0),
    Scale1(scale1),
    Space1(space1),
    Scale2(scale2),
    Space2(space2)
{
 parent = Parent.hwnd;
 owner  = Parent.hwnd;
 style  = Style;
}


OSlider::OSlider(const pOFrame Parent,
                 const ULONG scale1,
                 const ULONG space1,
                 const ULONG scale2,
                 const ULONG space2,
                 const ULONG Style)
 : OWindow(Parent->res, 0, 0),
   Scale1(scale1),
   Space1(space1),
   Scale2(scale2),
   Space2(space2)
{
 parent = Parent->hwnd;
 owner  = Parent->hwnd;
 style  = Style;
}


// dtor
OSlider::~OSlider() {}


BOOL OSlider::createSlider(const SHORT x, const SHORT y, 
                           const SHORT cx, const SHORT cy)
{
 ctlData.cbSize = sizeof(SLDCDATA);
 ctlData.usScale1Increments = Scale1;
 ctlData.usScale1Spacing = Space1;
 ctlData.usScale2Increments = Scale2;
 ctlData.usScale2Spacing = Space2;

 if (!hwnd) {
   hwnd = WinCreateWindow(parent, WC_SLIDER, NULL, style | WS_VISIBLE,
                          x, y, cx, cy,
                          owner, HWND_TOP, 0, (PVOID)&ctlData, NULL); }

 return (hwnd != NULLHANDLE);
}


BOOL OSlider::inherit(const HWND Parent)
{
 return(OWindow::inherit(Parent) && createSlider());
}


void OSlider::setSliderStyles(const ULONG Style)
{
  style = Style;
}


void OSlider::setSliderSize(const ULONG x, const ULONG y, 
                            const ULONG cx, const ULONG cy)
{
  sizepos.x = x;
  sizepos.y = y;
  sizepos.cx = cx;
  sizepos.cy = cy;
  WinSetWindowPos(hwnd, HWND_TOP, x, y, cx, cy, SWP_SIZE);
}


BOOL OSlider::setSliderArmSize(ULONG x, ULONG y)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSLIDERINFO,
                         MPFROMSHORT (SMA_SLIDERARMDIMENSIONS),
                         MPFROM2SHORT (x,y)));
}

BOOL OSlider::setSliderArmPos(SHORT pos)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSLIDERINFO,
                         MPFROM2SHORT(SMA_SLIDERARMPOSITION,SMA_INCREMENTVALUE),
                         MPFROMSHORT(pos)));
}

BOOL OSlider::setSliderShaftPos(SHORT x, SHORT y)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSLIDERINFO,
                         MPFROMSHORT (SMA_SHAFTPOSITION),
                         MPFROM2SHORT (x, y)));
}

BOOL OSlider::setSliderShaftDim(SHORT dim)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSLIDERINFO,
                         MPFROMSHORT (SMA_SHAFTDIMENSIONS),
                         MPFROMSHORT (dim)));
}

BOOL OSlider::setSliderScaleText(PCSZ Text, USHORT pos)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSCALETEXT,
                         MPFROMSHORT (pos),MPFROMP(Text)));
}

BOOL OSlider::setTickSize(ULONG size, ULONG num)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETTICKSIZE,
                         MPFROM2SHORT (num,size), NULL));
}

ULONG OSlider::addDetent(SHORT pos)
{
 return((ULONG) WinSendMsg(hwnd, SLM_ADDDETENT, MPFROMSHORT (pos), 0));
}

ULONG OSlider::getDetentPos(ULONG id)
{
 return(LONGFROMMP(WinSendMsg(hwnd, SLM_QUERYDETENTPOS, MPFROMLONG(id), 0)));
}


PSZ OSlider::getScaleText(SHORT loc, PSZ buf,SHORT len)
{
 WinSendMsg(hwnd, SLM_QUERYSCALETEXT,
            MPFROM2SHORT(loc,len),MPFROMP(buf));
 return((PSZ)buf);
}

void OSlider::getSliderShaftDim(SHORT *lenght, SHORT *height)
{
 ULONG  tmp;

 tmp = LONGFROMMP(WinSendMsg(hwnd,SLM_QUERYSLIDERINFO,
                             MPFROM2SHORT(SMA_SHAFTDIMENSIONS,SMA_INCREMENTVALUE),0));
 *lenght = SHORT1FROMMP(tmp);
 *height = SHORT2FROMMP(tmp);
}

void OSlider::getSliderShaftPos(SHORT *x, SHORT *y)
{
 ULONG   tmp;

 tmp = LONGFROMMP(WinSendMsg(hwnd,SLM_QUERYSLIDERINFO,
                             MPFROM2SHORT(SMA_SHAFTPOSITION,SMA_INCREMENTVALUE),0));
 *x = SHORT1FROMMP(tmp);
 *y = SHORT2FROMMP(tmp);
}

void OSlider::getSliderArmSize(SHORT *x, SHORT *y)
{
 ULONG   tmp;

 tmp = LONGFROMMP(WinSendMsg(hwnd,SLM_QUERYSLIDERINFO,
                             MPFROM2SHORT(SMA_SLIDERARMDIMENSIONS,SMA_INCREMENTVALUE),0));
 *x = SHORT1FROMMP(tmp);
 *y = SHORT2FROMMP(tmp);
}

ULONG OSlider::getSliderArmPos()
{
 return((ULONG)WinSendMsg(hwnd,SLM_QUERYSLIDERINFO,
                          MPFROM2SHORT(SMA_SLIDERARMPOSITION,SMA_INCREMENTVALUE),0));
}

ULONG OSlider::getTickSize(SHORT pos)
{
 return((ULONG)WinSendMsg(hwnd,SLM_QUERYTICKSIZE, MPFROMSHORT(pos), 0));
}

BOOL OSlider::incSliderArm(SHORT delta)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSLIDERINFO,
              MPFROM2SHORT(SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE),
              MPFROMSHORT((ULONG)WinSendMsg(hwnd, SLM_QUERYSLIDERINFO,
              MPFROM2SHORT(SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE),NULL) + delta)));
}

BOOL OSlider::decSliderArm(SHORT delta)
{
 return((BOOL)WinSendMsg(hwnd, SLM_SETSLIDERINFO,
              MPFROM2SHORT (SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE),
              MPFROMSHORT((ULONG)WinSendMsg(hwnd, SLM_QUERYSLIDERINFO,
              MPFROM2SHORT(SMA_SLIDERARMPOSITION, SMA_INCREMENTVALUE),NULL) - delta)));
}



void OSlider::drawSliderElement(POWNERITEM  item, COLOR color)
{
 WinQueryWindowRect(item->hwnd, &item->rclItem);
 GpiSetColor(item->hps, color);
 GpiBox(item->hps, DRO_FILL, (PPOINTL) &item->rclItem.xRight, 0L, 0L);
}

// end of source
