// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OSortedListBase.cpp


#include <ocl.hpp>
#include <OSortedListBase.hpp>


OSortedListBase::OSortedListBase() 
  : OMuxSem((PSZ)NULL),
    sortAscending(TRUE), 
    slistCursor(0) 
  {}


OSortedListBase::~OSortedListBase()
{
 requestMuxSem();
 releaseMuxSem();
}


void OSortedListBase::setAscending(BOOL ascending)
{
 sortAscending = ascending;
}


void OSortedListBase::addSorted(PVOID elem)
{
 if (!first){
   addFirst(elem);
   return; }

 if (sortAscending)
   addSortAscending(elem);
 else
   addSortDescending(elem);
}


void OSortedListBase::addSortAscending(PVOID elem)
{
 ULONG   step = 2, i;
 PPVITEM tmp;

 if (_isLess(elem, first->item))
  {
   addFirst(elem);
   return; 
  }

 if (_isLess(last->item, elem))
  {
   addLast(elem);
   return; 
  }

 requestMuxSem();
 slistCursor = items/step;

 tmp = OListBase::_getNextListItem(OListBase::_getItem(slistCursor));

 while(TRUE)
  {
   step = 2*step;
   slistCursor = items/step;
   if (slistCursor > 2)
     {
      if (_isLess(elem, tmp->item))
        {
         for(i = 0; i < slistCursor; i+=2)
            tmp = tmp->prev->prev; 
        }
      else 
        {
         for(i = 0; i < slistCursor; i+=2)
            tmp = tmp->next->next; 
        }
     }
   else break;
 }

 while (_isLess(elem, tmp->item))
    tmp = tmp->prev;

 while(_isLess(tmp->item, elem))
    tmp = tmp->next;

 addAfter(tmp->prev, elem);

 releaseMuxSem();
}


void OSortedListBase::addSortDescending(PVOID elem)
{
 ULONG    step = 2, i;
 PPVITEM  tmp;

 if (_isLess(first->item, elem))
  {
   addFirst(elem);
   return; 
  }

 if (_isLess(elem, last->item))
  {
   addLast(elem);
   return; 
  }

 requestMuxSem();
 slistCursor = items/step;

 tmp = OListBase::_getNextListItem(OListBase::_getItem(slistCursor));

 while(TRUE)
  {
   step = 2*step;
   slistCursor = items/step;
   if (slistCursor > 2)
     {
      if (_isLess(tmp->item, elem))
        {
         for(i = 0; i < slistCursor; i+=2)
            tmp = tmp->prev->prev; 
        }
      else 
        {
         for(i = 0; i < slistCursor; i+=2)
            tmp = tmp->next->next; 
        }
     }
   else break;
 }

 while (_isLess(tmp->item, elem))
    tmp = tmp->prev;

 while(_isLess(elem, tmp->item))
    tmp = tmp->next;

 addAfter(tmp->prev, elem);

 slistCursor = 0;
 releaseMuxSem();
}


// end of source
