// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OStatusLine.cpp

#define OINCL_OSTATLINE

#include <ocl.hpp>

OStatusLine::OStatusLine(const ULONG id, const HWND parentWindow)
  : OFrame(id, FCF_NOBYTEALIGN, CS_SIZEREDRAW)
{
 setParent(parentWindow);
 initStatlineDefaults();
}


OStatusLine::OStatusLine(const OFrame& parentWindow)
  : OFrame(parentWindow.res, FCF_NOBYTEALIGN, CS_SIZEREDRAW)
{
 setParent(parentWindow.hwnd);
 initStatlineDefaults();
}


OStatusLine::OStatusLine(const pOFrame parentWindow)
  : OFrame(parentWindow->res, FCF_NOBYTEALIGN, CS_SIZEREDRAW)
{
 setParent(parentWindow->hwnd);
 initStatlineDefaults();
}


OStatusLine::~OStatusLine() {}

void OStatusLine::initStatlineDefaults()
{
 orientation = DT_LEFT | DT_VCENTER;
 readOnly = FALSE;
 statlineText << (PSZ) "";
 pparms.Fore = SYSCLR_WINDOWTEXT;
 pparms.Back = SYSCLR_WINDOW;
}

BOOL OStatusLine::createStatusLine()
{
 if (!createFrame("OCL StatusLine"))
   return(FALSE);
 else {
   setFont("8.Helv");
   showFrame();
   return(TRUE); }
}


void OStatusLine::refreshText()
{
 WinInvalidateRect(hwnd, 0, TRUE);
}


void OStatusLine::setText(PCSZ text)
{
 if ((text) && (!readOnly)) {
   statlineText << (PSZ) " ";
   statlineText + (PSZ) text;
   statlineText + (PSZ) " ";
   refreshText(); }
}


ULONG OStatusLine::getHeight()
{
 ULONG statlineHeight;

 hps = WinGetPS(client);
 WinQueryWindowRect(HWND_DESKTOP, &rcl);
 WinDrawText(hps, 1, " ", &rcl, 0L, 0L, orientation | DT_QUERYEXTENT);
 WinFillRect(hps, &rcl, pparms.Back);
 WinReleasePS(hps);
 statlineHeight = rcl.yTop - rcl.yBottom+2;
 return(statlineHeight);
}


void OStatusLine::setOrientation(const ULONG Orientation) 
{
 orientation = Orientation; 
}


void OStatusLine::setRDOnly(BOOL readonly) 
{
 readOnly = readonly; 
}


BOOL OStatusLine::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
   {
    case WM_PAINT:
      hps = WinBeginPaint(hwnd, 0, 0);
      GpiCreateLogColorTable(hps, LCOL_RESET, LCOLF_RGB, 0L, 0L, NULL);
      WinQueryWindowRect(hwnd, &rcl);
      GpiSetColor(hps, pparms.Back);
      GpiBox(hps, DRO_FILL, (PPOINTL) &rcl.xRight, 0L, 0L);
      WinDrawText(hps, strlen(statlineText.text), statlineText.text, &rcl,
                       pparms.Fore, pparms.Back, orientation | DT_ERASERECT);
      WinEndPaint(hps);
      break;

    default:
      return(OFrame::OCommand(msg, mp1, mp2));
   }
 return(TRUE);

#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif


// end of source  
