// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OSysInfoVIO.cpp
// System Utility Functions

// class member functions


#define OINCL_OSYSINFO
#define OINCL_BASE

#define INCL_DOS


#include <ocl.hpp>
#include <iostream.h>
#include <fstream.h>

// Class OSysInfo


OSysInfoVIO::OSysInfoVIO() {}


OSysInfoVIO::~OSysInfoVIO()
{
 SysInfoList.reset();
}


BOOL OSysInfoVIO::getDrives()
{
 ULONG   DriveNumber;
 ULONG   LogicalDriveMap;
 UCHAR   Drive[] = "C:";
 ULONG   Temp;
 HFILE   hfDiskHandle;
 ULONG   Action;
 CHAR    LABEL[13], tLABEL[13];

 if (DosQueryCurrentDisk(&DriveNumber, &LogicalDriveMap))
    return(FALSE);

 for(Temp=(ULONG)Drive[0]-'A', LogicalDriveMap>>=2;
     Temp<=(ULONG)('Z'-'A');
     Temp++, Drive[0]++, LogicalDriveMap>>=1)
      {
       if(!(LogicalDriveMap&0x1))
          continue;

       if (!DosOpen((PSZ)Drive, &hfDiskHandle, &Action, 0, FILE_NORMAL,
                       OPEN_ACTION_OPEN_IF_EXISTS, OPEN_FLAGS_DASD |
                       OPEN_FLAGS_FAIL_ON_ERROR | OPEN_FLAGS_NOINHERIT |
                       OPEN_SHARE_DENYREADWRITE | OPEN_ACCESS_READONLY, 0))
           {
            struct _PPF
             {
              BYTE        bCommandInformation;
              BYTE        bDriveUnit;
             } PPF={0, 0};

            struct _DDF
             {
              BYTE        bData;
             } DDF;

            ULONG       ulParamLengthInOut=sizeof(PPF);
            ULONG       ulDataLengthInOut=sizeof(DDF);

            DosDevIOCtl(hfDiskHandle, IOCTL_DISK, DSK_BLOCKREMOVABLE,
                &PPF, ulParamLengthInOut, &ulParamLengthInOut,
                &DDF, ulDataLengthInOut, &ulDataLengthInOut);
            DosClose(hfDiskHandle);
            if(DDF.bData)
              {
               if (!DosQueryFSInfo(Drive[0]-'@', FSIL_VOLSER, tLABEL, sizeof(tLABEL)))
                sprintf(LABEL, " [%s]", &tLABEL[5]);
              else
                strcpy(LABEL, " [ ]"); 
             }
           else
              strcpy(LABEL, " [ ]");
           SysInfoVar << (PSZ) Drive;
           SysInfoVar + (PSZ) LABEL;
           SysInfoList << SysInfoVar.text;
          }
     }
 return(TRUE);
}


// check if "Drive" is a non-removable local drive

BOOL OSysInfoVIO::checkRemovable(PCSZ Drive)
{
 HFILE    hfDiskHandle;
 ULONG   Action;
 OString  drive((PSZ)Drive);

 cerr << drive.text;
 if (!DosOpen(drive.text, &hfDiskHandle, &Action, 0, FILE_NORMAL,
                   OPEN_ACTION_OPEN_IF_EXISTS, OPEN_FLAGS_DASD |
                   OPEN_FLAGS_FAIL_ON_ERROR | OPEN_FLAGS_NOINHERIT |
                   OPEN_SHARE_DENYREADWRITE | OPEN_ACCESS_READONLY, 0))
   {
    struct _PPF
      {
       BYTE        bCommandInformation;
       BYTE        bDriveUnit;
      } PPF={0, 0};

    struct _DDF
      {
       BYTE        bData;
      } DDF;

     ULONG       ulParamLengthInOut=sizeof(PPF);
     ULONG       ulDataLengthInOut=sizeof(DDF);

     DosDevIOCtl(hfDiskHandle, IOCTL_DISK, DSK_BLOCKREMOVABLE,
                      &PPF, ulParamLengthInOut, &ulParamLengthInOut,
                      &DDF, ulDataLengthInOut, &ulDataLengthInOut);
                      DosClose(hfDiskHandle);
     if (DDF.bData) return(TRUE);
  }
 return(FALSE);
}


// get var from system environment

BOOL OSysInfoVIO::getEnv(PCSZ env)
{
 PSZ Buffer = new CHAR[10*CCHMAXPATH];

 if(DosScanEnv(env, (PCSZ *) &Buffer))
   {
    delete[] Buffer;
    return(FALSE);
   }
 else
   {
    SysInfoVar << Buffer;
    delete[] Buffer;
    return(TRUE);
   }
}


// get boot drive

BOOL OSysInfoVIO::getBootDrive()
{
 ULONG      ulDrive;
 UCHAR      Buffer[10];

 memset(&Buffer, 0, sizeof(Buffer));

 if(DosQuerySysInfo(QSV_BOOT_DRIVE, QSV_BOOT_DRIVE, &ulDrive, sizeof(ulDrive)))
   return(FALSE);
 else
   {
    Buffer[0] = (char) (ulDrive + 'A' - 1 ) ;
    Buffer[1] = ':';
    Buffer[2] = '\0';
    SysInfoVar << (PSZ) Buffer;
    return(TRUE);
   }
}


// get an env string from config.sys

BOOL OSysInfoVIO::getCfgEnv(PSZ env)
{
 ifstream config;
 PSZ       Buffer;

 if (DosAllocMem((PPVOID) &Buffer,  0x0000ffffL, PAG_READ | PAG_WRITE | PAG_COMMIT))
   return(FALSE);

 if (getBootDrive())
   {
    SysInfoVar + (PSZ) "\\CONFIG.SYS";
    config.open(SysInfoVar.text);
    if (config.rdbuf())
     {
      while (config.getline(Buffer, 0x0000ffffL))
       {
        if (strstr(strupr(Buffer), strupr(env)))
         {
          config.close();
          SysInfoVar << Buffer;
          DosFreeMem(Buffer);
          return(TRUE);
         }
       }
      config.close();
     }
   }

 DosFreeMem(Buffer);

 return(FALSE);
}


// find a file in path

BOOL OSysInfoVIO::findPathFile(PCSZ file)
{
 BYTE  Buffer[CCHMAXPATH];

 if (getEnv("PATH")) {
   if (!DosSearchPath(SEARCH_CUR_DIRECTORY, SysInfoVar.text, (PSZ) file, Buffer, sizeof(Buffer))) {
     SysInfoVar << (PSZ) Buffer;
     return(TRUE); }}
 return(FALSE);
}

// get OS/2-Version

BOOL OSysInfoVIO::getOSVer()
{
 ULONG major;
 ULONG minor;
 CHAR  Buffer[10];

 if ((!DosQuerySysInfo(QSV_VERSION_MAJOR, QSV_VERSION_MAJOR, &major, sizeof(major))) &&
     (!DosQuerySysInfo(QSV_VERSION_MINOR, QSV_VERSION_MINOR, &minor, sizeof(minor))) )
  {
   sprintf(Buffer, "%ld.%ld\0", major/10, minor);
   SysInfoVar << (PSZ) Buffer;
   return(TRUE);
  }
 return(FALSE);
}


BOOL OSysInfoVIO::getOSVer(double& version)
{
 ULONG major;
 ULONG minor;

 if ((!DosQuerySysInfo(QSV_VERSION_MAJOR, QSV_VERSION_MAJOR, &major, sizeof(major))) &&
     (!DosQuerySysInfo(QSV_VERSION_MINOR, QSV_VERSION_MINOR, &minor, sizeof(minor))) )
  {
   version = minor;
   version /= 100;
   version += major/10;
   return(TRUE);
  }
 return(FALSE);
}


BOOL OSysInfoVIO::clearRDOnly(PCSZ FileName)
{
 FILESTATUS  FileInfoBuf;
 APIRET        rc=3;

 if (!DosQueryPathInfo((PSZ) FileName, 1, &FileInfoBuf, sizeof(FILESTATUS)))
   {
    FileInfoBuf.attrFile = FILE_NORMAL;
#ifdef __BCPLUSPLUS__
    rc = DosSetPathInfo((PSZ)FileName, 1, &FileInfoBuf, sizeof(FILESTATUS), DSPI_WRTTHRU);
#else
    rc = DosSetPathInfo(FileName, 1, &FileInfoBuf, sizeof(FILESTATUS), DSPI_WRTTHRU);
#endif
   }
 return((rc==0) || (rc==2));
}


// end of source
