// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OSystem.cpp


#define OINCL_OPROCSTAT
#define OINCL_OSORTEDLIST
#define OINCL_BASE

#define INCL_DOSPROCESS
#define INCL_DOSMEMMGR

#include <ocl.hpp>


#define PROCSTAT_PID  1
#define PROCSTAT_PPID 2

extern "C" 
  {
#ifdef __IBMCPP__
   USHORT APIENTRY16 DOSQPROCSTATUS(PVOID _Seg16 pvBuf, USHORT cbBuf);
#elif __WATCOM_CPLUSPLUS__
   USHORT APIENTRY16 DOSQPROCSTATUS(PVOID __far16 pvBuf, USHORT cbBuf);
#elif __BCPLUSPLUS__
   USHORT APIENTRY16 DOSQPROCSTATUS(PVOID __far16 pvBuf, USHORT cbBuf);
#endif
  }


// member functions of OProcRecord


// copy constructor

OProcRecord::OProcRecord(const pOProcRecord newRecord)
    : Threads(newRecord->Threads), Pid(newRecord->Pid),
      PPid(newRecord->PPid), Sid(newRecord->Sid)
{
 Module << newRecord->Module.text;
}

OProcRecord::OProcRecord(const OProcRecord& newRecord)
    : Threads(newRecord.Threads), Pid(newRecord.Pid),
      PPid(newRecord.PPid), Sid(newRecord.Sid)
{
 Module << newRecord.Module.text;
}

OProcRecord::OProcRecord() {}

OProcRecord::~OProcRecord() {}


// member funtions of OProcStat

OProcStat::OProcStat()
  : ProcCount(0), TaskCount(0) {};

OProcStat::~OProcStat()
{
 ProcList.reset();
}


BOOL OProcStat::getProcesses()
{
 OProcRecord record;
 PBUFFHEADER pbh;     //  processinfo

// clean up

  ProcList.reset();
  ProcCount = 0;
  TaskCount = 0;

// Allocate memory for the process-status information structure returned by OS/2.

 if (DosAllocMem((PPVOID)&pbh,  PROCSTAT_BUFFER_SIZE,
                 PAG_READ | PAG_WRITE | PAG_COMMIT | OBJ_TILE))
   return(FALSE);

// undocumented API DosQProcStatus to fill the information structure.

 

#ifdef __IBMCPP__
 if(DOSQPROCSTATUS((PVOID _Seg16)pbh, PROCSTAT_BUFFER_SIZE))
#elif __WATCOM_CPLUSPLUS__
 if(DOSQPROCSTATUS((PVOID __far16)pbh, PROCSTAT_BUFFER_SIZE))
#elif __BCPLUSPLUS__
 if(DOSQPROCSTATUS((PVOID __far16)pbh, PROCSTAT_BUFFER_SIZE))
#endif
  {
   DosFreeMem(pbh);
   return(FALSE); 
  }
 else
  {
   PMODINFO  pmi = pbh->pmi;         // Start at first module

   while(pmi)
    {
     PPROCESSINFO    ppi=pbh->ppi;           // Start at first process

     while(ppi->ulEndIndicator != PROCESS_END_INDICATOR)
       {
        if(ppi->hModRef==pmi->hMod)       // Module found
         {
          if (strcmp(pmi->szModName, "SYSINIT") == 0)
            record.Module << (PSZ) "MDOS Process";
          else
            record.Module << (PSZ) strupr(pmi->szModName);
          record.Pid = ppi->pid;
          record.PPid = ppi->pidParent;
          record.Sid = ppi->idSession;
          record.Threads = ppi->usThreadCount;
          ProcList << &record;
          TaskCount++;
         }
        ppi=(PPROCESSINFO)(ppi->ptiFirst+ppi->usThreadCount);       // point to next process
       }
     ProcCount++;
     pmi=pmi->pNext;                     // Next module
    }
  }
 DosFreeMem(pbh);
 return(TRUE);
}



ULONG OProcStat::getPid(PSZ module)
{
 return(getNumInfo(module, PROCSTAT_PID));
}


ULONG OProcStat::getPPid(PSZ module)
{
 return(getNumInfo(module, PROCSTAT_PPID));
}


ULONG OProcStat::getNumInfo(PSZ module, ULONG flag)
{
 OString               tmpString;
 ListItem<OProcRecord> *tmp;

 if (!getProcesses())
   return(0);

 tmp = (ListItem<OProcRecord>*) ProcList.first;
 tmpString << module;

 while((tmp) && (tmp->item))
  {
   if (tmpString == tmp->item->Module)
    {
     switch (flag)
      {
       case PROCSTAT_PID:
         return(tmp->item->Pid);

       case PROCSTAT_PPID:
         return(tmp->item->PPid);
      }     
    }
   tmp = tmp->next;
  }
 return(0);
}


PSZ OProcStat::getModule(ULONG pid)
{
 ListItem<OProcRecord> *tmp;

 if (!getProcesses())
   return(0);

 tmp = (ListItem<OProcRecord>*) ProcList.first;

 while((tmp) && (tmp->item))
  {
   if (tmp->item->Pid == pid)
     return(strdup(tmp->item->Module.text));
   tmp = tmp->next;
  }
 return(NULL);
}


ULONG OProcStat::getProcCount() 
{
 if (getProcesses())
    return(ProcCount);
 else return(0); 
}


ULONG OProcStat::getTaskCount() 
{
 if (getProcesses())
    return(TaskCount);
 else return(0); 
}


slOProcRecord::slOProcRecord() {};
slOProcRecord::~slOProcRecord() {};

void  slOProcRecord::operator << (pOProcRecord newRecord)
{
 pOProcRecord tmp = new OProcRecord(newRecord);
 addSorted(tmp);
}


void slOProcRecord::output(PVOID elem)
{
 cout << "Module: " << ((OProcRecord*)elem)->Module.text << endl
       << "Threads: " << ((OProcRecord*)elem)->Threads << endl
       << "Process-ID: " << ((OProcRecord*)elem)->Pid << endl
       << "Parent-ID: " << ((OProcRecord*)elem)->PPid << endl
       << "Session-ID: " << ((OProcRecord*)elem)->Sid << endl << endl; 
}


BOOL slOProcRecord::isLess(const OProcRecord *first, const OProcRecord *second)
{
 return(first->Pid < second->Pid);
}


// end of source
