// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OWindow.cpp


#define OINCL_OWINDOW

#include <ocl.hpp>


OWindow::OWindow(const ULONG id, const ULONG winstyle, const ULONG winbits)
   : termFlag(FALSE),
     parent(HWND_DESKTOP),
     owner(NULLHANDLE),
     res(id),
     style(winstyle),
     bits(winbits),
     hps(0)
{
 memset(&sizepos, 0, sizeof(SWP));
}




BOOL OWindow::inherit(const HWND hwndDlg)
{
 hwnd = WinWindowFromID(hwndDlg, res);
 parent = hwndDlg;
 owner = hwndDlg;
 return((hwnd != NULLHANDLE) &&
        (hwndDlg != NULLHANDLE));
}


OWindow::~OWindow() 
{ 
 destroy(); 
}

void OWindow::show()
{
 if (hwnd)
    WinSetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_RESTORE | SWP_ACTIVATE);
}

void OWindow::hide()
{
 if (hwnd)
    WinSetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_HIDE | SWP_DEACTIVATE);
}

void OWindow::activate()
{
 if (hwnd)
    WinSetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_SHOW | SWP_ACTIVATE);
}


void OWindow::enable()
{
 if (hwnd)
    WinEnableWindow(hwnd, TRUE);
}


void OWindow::disable()
{
 if (hwnd)
    WinEnableWindow(hwnd, FALSE);
}


void OWindow::destroy()
{
 if (hwnd)
    WinDestroyWindow(hwnd);
}

void OWindow::setFocus()
{
 if (hwnd)
    WinSetFocus(HWND_DESKTOP, hwnd);
}


void OWindow::setParent(const HWND handle)
{
 parent = handle; 
 WinSetParent(hwnd, parent, TRUE);
}


void OWindow::setOwner(const HWND handle)
{
 owner = handle;
 WinSetOwner(hwnd, owner);
}

void OWindow::setForeColor(COLOR color)
{
 pparms.Fore = color;
 WinSetPresParam(hwnd, PP_FOREGROUNDCOLOR, sizeof(COLOR), &pparms.Fore);
 WinUpdateWindow(hwnd);
}


void OWindow::setBackColor(COLOR color)
{
 pparms.Back = color;
 WinSetPresParam(hwnd, PP_BACKGROUNDCOLOR, sizeof(COLOR), &pparms.Back);
 WinUpdateWindow(hwnd);
}


void OWindow::setFont(PSZ fontnamesize)
{
 strcpy(pparms.Font, fontnamesize);
 WinSetPresParam(hwnd, PP_FONTNAMESIZE, strlen(pparms.Font)+1, pparms.Font);
 WinUpdateWindow(hwnd);
}



void OWindow::setPresentation(pPresParms presentation)
{
 pparms.Fore = presentation->Fore;
 pparms.Back = presentation->Back;
 WinSetPresParam(hwnd, PP_FOREGROUNDCOLOR, sizeof(COLOR), &pparms.Fore);
 WinSetPresParam(hwnd, PP_BACKGROUNDCOLOR, sizeof(COLOR), &pparms.Back);
 setFont(presentation->Font);
}


COLOR OWindow::getForeColor()
{
 ULONG ppid;

 WinQueryPresParam(hwnd, PP_FOREGROUNDCOLOR, 0, &ppid,
                   sizeof(COLOR), &pparms.Fore, 0);
 return(pparms.Fore);
}



COLOR OWindow::getBackColor()
{
 ULONG ppid;

 WinQueryPresParam(hwnd, PP_BACKGROUNDCOLOR, 0, &ppid,
                   sizeof(COLOR), &pparms.Back, 0);
 return(pparms.Back);
}


PSZ OWindow::getFont()
{
 ULONG ppid;

 WinQueryPresParam(hwnd, PP_FONTNAMESIZE, 0, &ppid, sizeof(pparms.Font), &pparms.Font, 0);
 return(pparms.Font);
}

void OWindow::getPresentation(pPresParms presentation)
{
 presentation->Fore = getForeColor();
 presentation->Back = getBackColor();
 strcpy(presentation->Font, getFont());
}


BOOL OWindow::initReplace(const ULONG id, SWP& swp)
{
 HWND tmp = WinWindowFromID(parent, id);
 
 if (tmp) {
   WinQueryWindowPos(tmp, &swp);
   WinDestroyWindow(tmp);
   return(TRUE); }
 return(FALSE);
}

// end of source
