// OCL - Class Library for OS/2 2.x and up
// (c) Benjamin Stein 1994
// All Rights Reserved
// OXcptBase.cpp


#include <ocl.hpp>
#include <OXcptBase.hpp>


pOXcptBase OXcptBase::currentHandler = NULL;


OXcptBase::OXcptBase(BOOL killMessages)
   : killMsg(killMessages),
     record(NULL) {}

OXcptBase::~OXcptBase() {}



BOOL OXcptBase::setHandler(OXCPTRECORD& regRecord)
{
 if (!OXcptBase::currentHandler)
   OXcptBase::currentHandler = this;

 record = &regRecord;

 regRecord.prev_structure   = NULL;
 regRecord.ExceptionHandler = (_ERR*)&OXcptBase::OXCPT_CATCH;

 if (!DosSetExceptionHandler(&regRecord))
   return(!DosSetSignalExceptionFocus(SIG_SETFOCUS, &times));
 return(FALSE);
}


BOOL OXcptBase::unsetHandler()
{
 if (OXcptBase::currentHandler == this)
   OXcptBase::currentHandler = NULL;
 
 if (record)
   return(!DosUnsetExceptionHandler(record));
 return(FALSE);
}



BOOL OXcptBase::handler(PEXCEPTIONREPORTRECORD report,
                        PCONTEXTRECORD context,
                        PVOID pargs)

{
 OString  msg;

 msg << "\nException Handler registered:\n";

 switch(report->ExceptionNum)
  {
   case XCPT_GUARD_PAGE_VIOLATION:
    msg + "Guard Page Violation. Try to continue...\n";
    exceptionMessage(msg);
    return(TRUE);

   case XCPT_UNABLE_TO_GROW_STACK:
    msg + "Unable to grow stack.";
    break;

   case XCPT_INTEGER_DIVIDE_BY_ZERO:
   case XCPT_FLOAT_DIVIDE_BY_ZERO:
    msg + "Divide by Zero.";
    break;

   case XCPT_FLOAT_INVALID_OPERATION:
    msg + "Invalid float operation.";
    break;

   case XCPT_ILLEGAL_INSTRUCTION:
    msg + "Illegal instruction.";
    break;

   case XCPT_PRIVILEGED_INSTRUCTION:
    msg + "Privileged instruction.";
    break;

   case XCPT_INTEGER_OVERFLOW:
    msg + "Integer overflow.";
    break;

   case XCPT_FLOAT_OVERFLOW:
   case XCPT_FLOAT_UNDERFLOW:
    msg + "Float underflow or overflow.";
    break;

   case XCPT_FLOAT_DENORMAL_OPERAND:
    msg + "Denormal operand in arithmetic operation.";
    break;

   case XCPT_FLOAT_INEXACT_RESULT:
    msg + "Inexact result in arithmetic operation.";
    break;

   case XCPT_FLOAT_STACK_CHECK:
    msg + "Invalid float stack check.";
    break;

   case XCPT_DATATYPE_MISALIGNMENT:
    msg + "Data type misalignment.\n";
    switch(report->ExceptionInfo[0])
    {
     case XCPT_READ_ACCESS:
       msg + "Flag: Read Access.";
       break;
     case XCPT_WRITE_ACCESS:
       msg + "Flag: Write Access.";
       break;
    }
    break;

   case XCPT_IN_PAGE_ERROR:
    msg + "Page Read Error.";
    break;

   case XCPT_NONCONTINUABLE_EXCEPTION:
    msg + "Non-continuable exception.";
    break;

   case XCPT_INVALID_DISPOSITION:
    msg + "Invalid Disposition.";
    break;

   case XCPT_ARRAY_BOUNDS_EXCEEDED:
    msg + "Array bounds exceeded.";
    break;

   case XCPT_BAD_STACK:
    msg + "Bad stack.";
    break;

   case XCPT_INVALID_UNWIND_TARGET:
    msg + "Invalid unwind target.";
    break;

   case XCPT_ACCESS_VIOLATION:
    msg + "Memory Access Violation.\n";
    switch(report->ExceptionInfo[0])
     {
      case XCPT_UNKNOWN_ACCESS:
        msg + "Flag: Unknown Access.";
        break;
      case XCPT_READ_ACCESS:
        msg + "Flag: Read Access.";
        break;
      case XCPT_WRITE_ACCESS:
        msg + "Flag: Write Access.";
        break;
      case XCPT_EXECUTE_ACCESS:
        msg + "Flag: Execute Access.";
        break;
      case XCPT_SPACE_ACCESS:
        msg + "Flag: Space Access.";
        break;
      case XCPT_LIMIT_ACCESS:
        msg + "Flag: Limit Access.";
        break;
      default:
        msg + "Flag: unkown flag.";
        break;
     }
    break;

   case XCPT_SIGNAL:
    msg + "System Signal Exception.\n";
    switch(report->ExceptionInfo[0])
     {
      case XCPT_SIGNAL_KILLPROC:
        msg + "External DosKillProcess.";
        break;
      case XCPT_SIGNAL_INTR:
        msg + "CTRL+C.";
        break;
      case XCPT_SIGNAL_BREAK:
        msg + "CTRL+BREAK.";
        break;
     }
    if (killMsg)
      exceptionMessage(msg);
    return(killTrace());

   default:
     return(FALSE);
  }

 exceptionMessage(msg);

 return(trapTrace());
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif



ULONG OXcptBase::OXCPT_CATCH(EXCEPTIONREPORTRECORD *report,
                             OXCPTRECORD *record,
                             CONTEXTRECORD *context,
                             PVOID pargs)
{
 if ((!OXcptBase::currentHandler) || 
     (!OXcptBase::currentHandler->handler(report, context, pargs)))
   return(XCPT_CONTINUE_SEARCH);
 else
   return(XCPT_CONTINUE_EXECUTION);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif


// end of source
